package bb;

import java.util.ArrayList;

import remoteInterfaces.BulletinBoard;
import votingSystem.BinaryVote;
import votingSystem.DecryptedShare;
import votingSystem.MultipleBinaryVotes;
import votingSystem.MultipleVotes;
import votingSystem.Vote;

/**
 * Implementation of the remote class BulletinBoard that is bound to the RMI
 * registry for remote use. The BulletinBoard holds the votes and the
 * DecryptedShares for the election
 * 
 * @author Bent K. Onshus
 * 
 */
public class BulletinBoard_Impl implements BulletinBoard {

	private ArrayList votes = new ArrayList();

	private ArrayList binaryVotes = new ArrayList();

	private ArrayList multipleVotes = new ArrayList();

	private ArrayList multipleBinaryVotes = new ArrayList();

	private ArrayList decryptedShares = new ArrayList();

	private ArrayList decryptedShareList = new ArrayList();

	private int noOfVotes = 0;

	private int noOfShares = 0;

	private boolean finished = false;

	/**
	 * Empty constructor.
	 * 
	 * 
	 */
	public BulletinBoard_Impl() {

	}

	/**
	 * Posts a Vote to the bulletin board
	 * 
	 * @param v
	 *            The vote
	 * 
	 */
	public void postVote(Vote v) throws java.rmi.RemoteException {
		if (!finished) {
			System.out.println(v.getVote());
			votes.add(v);
			noOfVotes++;
		}
	}

	/**
	 * Posts a BinaryVote to the bulletin board
	 * 
	 * @param v
	 *            The BinaryVote
	 * 
	 */
	public void postVote(BinaryVote v) throws java.rmi.RemoteException {
		if (!finished) {
			binaryVotes.add(v);
			noOfVotes++;
		}
	}

	/**
	 * Posts MultipleVotes to the bulletin board
	 * 
	 * @param v
	 *            The votes
	 * 
	 */
	public void postVote(MultipleVotes v) throws java.rmi.RemoteException {
		if (!finished) {
			multipleVotes.add(v);
			noOfVotes++;
		}
	}

	/**
	 * Posts MultipleBinaryVotes to the bulletin board
	 * 
	 * @param v
	 *            The votes
	 * 
	 */
	public void postVote(MultipleBinaryVotes v) throws java.rmi.RemoteException {
		if (!finished) {
			multipleBinaryVotes.add(v);
			noOfVotes++;
		}
	}

	/**
	 * Posts a DecryptedShare to the bulletin board
	 * 
	 * @param ds
	 *            The DecryptedShare
	 */
	public void postShare(DecryptedShare ds) throws java.rmi.RemoteException {
		noOfShares++;
		decryptedShares.add(ds);

	}

	/**
	 * Posts multiple DecryptedShares to the bulletin board in a multiple vote
	 * election
	 * 
	 * @param ds
	 *            The DecryptedShares
	 */
	public void postShare(DecryptedShare[] ds) throws java.rmi.RemoteException {
		noOfShares++;
		decryptedShareList.add(ds);
	}

	/**
	 * Gets the DecryptedShares in the case of a single vote election
	 * 
	 * @return The shares in an array
	 * 
	 */
	public DecryptedShare[] getOneOfLShares() throws java.rmi.RemoteException {
		return (DecryptedShare[]) decryptedShares
				.toArray(new DecryptedShare[decryptedShares.size()]);
	}

	/**
	 * Gets the DecryptedShares in the case of a multiple vote election
	 * 
	 * @return The shares as DecryptedShare-arrays in an ArrayList
	 * 
	 */
	public ArrayList getKOfLShares() throws java.rmi.RemoteException {
		return decryptedShareList;
	}

	/**
	 * Gets the Votes in the case of a single candidate election
	 * 
	 * @return The Votes in an array
	 */
	public Vote[] getVotes() throws java.rmi.RemoteException {
		Vote[] temp = (Vote[]) votes.toArray(new Vote[votes.size()]);
		return temp;
	}

	/**
	 * Gets the BinaryVotes in the case of a single candidate election
	 * 
	 * @return The BinaryVotes in an array
	 */
	public BinaryVote[] getBinaryVotes() throws java.rmi.RemoteException {
		return (BinaryVote[]) binaryVotes.toArray(new BinaryVote[binaryVotes
				.size()]);
	}

	/**
	 * Gets the MultipleVotes in the case of a multi candidate election
	 * 
	 * @return The MultipleVotes in an array
	 */
	public MultipleVotes[] getMultipleVotes() throws java.rmi.RemoteException {
		return (MultipleVotes[]) multipleVotes
				.toArray(new MultipleVotes[multipleVotes.size()]);
	}

	/**
	 * Gets the MultipleBinaryVotes in the case of a multi candidate election
	 * 
	 * @return The MultipleBinaryVotes in an array
	 */
	public MultipleBinaryVotes[] getMultipleBinaryVotes()
			throws java.rmi.RemoteException {
		return (MultipleBinaryVotes[]) multipleBinaryVotes
				.toArray(new MultipleBinaryVotes[multipleBinaryVotes.size()]);
	}

	/**
	 * Gets the number of votes cast in the election
	 * 
	 * @return The number of votes
	 * 
	 */
	public int getNoOfVotes() throws java.rmi.RemoteException {
		return noOfVotes;
	}

	/**
	 * Gets the number of DecryptedShares posted by the DecryptionServers
	 * 
	 * @return The number of DecryptedShares
	 * 
	 */
	public int getNoOfShares() throws java.rmi.RemoteException {
		return noOfShares;
	}

	/**
	 * Ends the election, thus disallowing the posting of additional votes
	 * 
	 */
	public void endElection() throws java.rmi.RemoteException {
		finished = true;
	}

	/**
	 * Gets the status of the election
	 * 
	 * @return The status of the election. True if it is finished, false if not.
	 * 
	 */
	public boolean getElectionStatus() throws java.rmi.RemoteException {
		return finished;
	}

}
