package ds;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.security.SecureRandom;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.UIManager;

import remoteInterfaces.BulletinBoard;
import remoteInterfaces.ElectionServer;
import votingSystem.ElectionProperties;
import votingSystem.KeyShare;
import votingSystem.PublicKey;
import exceptions.NoValidVotesException;

/** The class starting the Decryption Server. This class specifies the user interface.
 * 
 * @author bent
 *
 */
public class DSGUI extends JFrame implements ActionListener {

	private Dimension screenSize;

	private JFrame dialog;

	private JTextField hostField;

	private JSpinner portField;

	private RemoteEntityConnectionDS rec;

	private BulletinBoard bb;

	private ElectionProperties ep;

	private KeyShare ks;

	private PublicKey pk;

	private ElectionServer es;

	private JLabel noVotesLabel2, finishedLabel2;

	private JButton calculateShareButton;

	private Server s;

	private Random rand;

	private int id;

	private JPanel bottomLayer;

	public DSGUI() {
		super("DecryptionServer");

		try {
			UIManager
					.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Exception e) {
		}

		this.setSize(800, 600);
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation(screenSize.width / 2 - 400,
				screenSize.height / 2 - 300);
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		rand = (Random) new SecureRandom();
		createStartupWindow();
	}

	public void createStartupWindow() {
		dialog = new JFrame();
		dialog.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel dialogPanel = new JPanel(new BorderLayout());
		JPanel inputPanel = new JPanel(new GridLayout(2, 2));
		dialog.setSize(500, 200);

		dialog.setLocation(screenSize.width / 2 - 250,
				screenSize.height / 2 - 100);
		dialog.setVisible(true);
		dialog.setResizable(false);
		JLabel hostLabel = new JLabel("IP/Hostname for Server");
		JLabel portLabel = new JLabel("TCP Port for Server");
		hostField = new JTextField("localhost");
		hostField.setHorizontalAlignment(JTextField.RIGHT);
		hostField.setColumns(20);
		portField = new JSpinner();
		portField.setValue(new Integer("2004"));
		((JSpinner.NumberEditor) portField.getEditor()).getTextField()
				.setColumns(17);

		JPanel hostLabelPanel = new JPanel();
		JPanel hostFieldPanel = new JPanel();
		JPanel portLabelPanel = new JPanel();
		JPanel portFieldPanel = new JPanel();

		hostLabelPanel.add(hostLabel);
		hostFieldPanel.add(hostField);
		portLabelPanel.add(portLabel);
		portFieldPanel.add(portField);

		inputPanel.add(hostLabelPanel);
		inputPanel.add(hostFieldPanel);
		inputPanel.add(portLabelPanel);
		inputPanel.add(portFieldPanel);

		JButton dialogButton = new JButton("Connect");
		dialogButton.addActionListener(this);

		dialogPanel.add(inputPanel);
		dialogPanel.add(dialogButton, BorderLayout.SOUTH);
		dialog.getContentPane().add(dialogPanel);

		dialog.validate();
	}

	public void createGUI() {
		bottomLayer = new JPanel(new BorderLayout());
		JPanel fieldLayer = new JPanel(new GridLayout(3, 2));
		JPanel topPanel = new JPanel();
		JLabel label = new JLabel("BulletinBoard up'n running!");
		this.getContentPane().add(bottomLayer);
		topPanel.add(label);
		bottomLayer.add(topPanel, BorderLayout.NORTH);

		JPanel noVotesLabelPanel = new JPanel();
		JPanel noVotesLabelPanel2 = new JPanel();
		JLabel noVotesLabel = new JLabel(
				"Number of Votes Given. Refresh to get updated value");
		noVotesLabel2 = new JLabel("0");
		noVotesLabelPanel.add(noVotesLabel);
		noVotesLabelPanel2.add(noVotesLabel2);
		fieldLayer.add(noVotesLabelPanel);
		fieldLayer.add(noVotesLabelPanel2);

		JPanel noSharesLabelPanel = new JPanel();
		JPanel noSharesLabelPanel2 = new JPanel();

		JLabel noSharesLabel = new JLabel(
				"Number of shares acquired acquired from Decryption Servers");
		JLabel noSharesLabel2 = new JLabel("0");
		noSharesLabelPanel.add(noSharesLabel);
		noSharesLabelPanel2.add(noSharesLabel2);
		fieldLayer.add(noSharesLabelPanel);
		fieldLayer.add(noSharesLabelPanel2);

		JPanel finishedLabelPanel = new JPanel();
		JPanel finishedLabelPanel2 = new JPanel();

		JLabel finishedLabel = new JLabel("Election finished?");
		finishedLabel2 = new JLabel("No");
		finishedLabelPanel.add(finishedLabel);
		finishedLabelPanel2.add(finishedLabel2);
		fieldLayer.add(finishedLabelPanel);
		fieldLayer.add(finishedLabelPanel2);

		bottomLayer.add(fieldLayer);

		JPanel buttonLayer = new JPanel();
		calculateShareButton = new JButton("Calculate Share");
		calculateShareButton.setEnabled(false);
		calculateShareButton.addActionListener(this);
		JButton refreshButton = new JButton("Refresh");
		refreshButton.addActionListener(this);

		buttonLayer.add(refreshButton);
		buttonLayer.add(calculateShareButton);

		bottomLayer.add(buttonLayer, BorderLayout.SOUTH);
		this.getContentPane().validate();

	}

	public void showSharePosted() {
		bottomLayer.removeAll();
		bottomLayer.add(new JLabel("Share Posted on BulletinBoard"));
		bottomLayer.validate();
	}

	public void calculateShare() {

		try {
			s.postShare();
			showSharePosted();
		} catch (RemoteException e) {
			e.printStackTrace();
			showDialog("Could not find BulletinBoard, or access denied!");
		} catch (NoValidVotesException e) {
			showDialog("No valid votes found");
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("Connect")) {
			String host = hostField.getText();
			int port = ((Integer) portField.getValue()).intValue();
			try {
				rec = new RemoteEntityConnectionDS(host, port);
				bb = rec.getBulletinBoard();
				es = rec.getElectionServer();
				pk = es.getPublicKey();
				ep = es.getElectionProperties();
				ks = es.getKeyShare();
				id = ks.getId();
				s = new Server(id, ks, ep, rand, pk, bb);
				dialog.dispose();
				createGUI();

			} catch (RemoteException e1) {
				showDialog("Could not connect to registry. Check host and port!");
			} catch (NotBoundException el) {
				showDialog("Could not locate remote object.");
			}

		} else if (e.getActionCommand().equals("Refresh")) {

			try {
				noVotesLabel2.setText(bb.getNoOfVotes() + "");

				if (bb.getElectionStatus()) {
					finishedLabel2.setText("Yes");
					calculateShareButton.setEnabled(true);
				} else {
					finishedLabel2.setText("No");
				}
			} catch (RemoteException e1) {
				showDialog("Could not contact Server");
			}
		} else if (e.getActionCommand().equals("Calculate Share")) {
			calculateShare();
		}
	}

	public void showDialog(String msg) {
		JOptionPane.showMessageDialog(this, msg);
	}

	public static void main(String[] args) {
		new DSGUI();
	}

}
