package ds;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import remoteInterfaces.BulletinBoard;
import remoteInterfaces.ElectionServer;

/** The class used for retrieving the stubs for the remote objects used by the decryption servers.
 * 
 * @author bent
 *
 */

public class RemoteEntityConnectionDS {

	private Registry registry;
/** Connects to the RMI registry at the given host and port.
 * 
 * @param host
 * @param port
 * @throws RemoteException If the RMI registry can't be found
 */
	public RemoteEntityConnectionDS(String host, int port)
			throws RemoteException {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new SecurityManager());
		}

		try {
			registry = LocateRegistry.getRegistry(host, port);

		} catch (RemoteException e) {
			throw e;
		}
	}
/** Gets the ElectionServer stub
 * 
 * @return The stub for the remote object ElectionServer
 * @throws AccessException If access was denied
 * @throws RemoteException 
 * @throws NotBoundException If the object can't be found at the registry
 */
	public ElectionServer getElectionServer() throws AccessException,
			RemoteException, NotBoundException {
		ElectionServer obj;

		obj = (ElectionServer) registry.lookup("ElectionServer");

		return obj;
	}
/** Gets the BulletinBoard stub
 * 
 * @return The stub for the remote object BulletinBoard
 * @throws AccessException If access was denied
 * @throws RemoteException
 * @throws NotBoundException If the object can't be found at the registry
 */
	public BulletinBoard getBulletinBoard() throws AccessException,
			RemoteException, NotBoundException {
		BulletinBoard obj;

		obj = (BulletinBoard) registry.lookup("BulletinBoard");

		return obj;
	}

}
