package ea;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.BigInteger;
import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.security.SecureRandom;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JWindow;
import javax.swing.UIManager;

import remoteInterfaces.BulletinBoard;
import remoteInterfaces.ElectionServer;
import votingSystem.ElectionProperties;
import votingSystem.KeyGenerator;
import votingSystem.KeyShare;
import exceptions.InsufficientSharesException;
/** The class that starts the election authority. This class mainly specifies the user interface.
 * 
 * @author bent
 *
 */
public class GUI extends JFrame implements ActionListener, Runnable {

	// Parameters for the Election
	private int keylength, t, M, L, l, w, type, k;

	private String[] voteNames;

	private int[] voteWeights;

	private KeyGenerator kg;

	private ElectionAuthority ea;

	private ElectionProperties ep;

	private BulletinBoard bb;

	private ElectionServer es;

	private Random rand;

	private RemoteEntityConnectionEA rec;

	private int port = 2004;

	private KeyShare[] ks;

	private JWindow progWin;

	private JPanel bottomLayer;

	// Boxes for pageOne
	private JSpinner nSpin;

	private JSpinner MSpin;

	private JSpinner kSpin;

	private JSpinner tSpin;

	private JSpinner LSpin;

	private JSpinner lSpin;

	private JSpinner wSpin;

	private JComboBox typeBox;

	// Boxes for pageTwo
	private JLabel[] weights;

	private JSpinner[] weightsSpin;

	private JTextField[] nameFields;

	// Boxes for pageThree
	private JLabel noServers;

	private JLabel noVotes;

	private JLabel noVotesCast;

	private JLabel noOfShares;

	// Progressbar message
	private String msg = "";
/**
 * Starts a new GUI for the election authority
 *
 */
	public GUI(int port ) {
		super("Election Authority");
		try {
			UIManager
					.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Exception e) {
		}
		this.port = port;
		bottomLayer = new JPanel(new BorderLayout());

		this.getContentPane().add(bottomLayer);
		this.setSize(800, 600);
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		createPageOne();
	}

	private void createPageOne() {
		JLabel pageOneText = new JLabel("Set up the election (1/2)");
		JPanel pageOne = new JPanel(new GridLayout(8, 2));
		bottomLayer.add(pageOneText, BorderLayout.NORTH);
		bottomLayer.add(pageOne);
		JLabel nLabel = new JLabel("Keylength of Public Key");
		JLabel MLabel = new JLabel("Maximum number of voters (M)");
		JLabel kLabel = new JLabel("Number of votes (t)");
		JLabel tLabel = new JLabel(
				"Security parameter. Length of bitchallenges (q)");
		JLabel LLabel = new JLabel("Number of candidates (L)");
		JLabel lLabel = new JLabel("Number of decryption servers (l)");
		JLabel wLabel = new JLabel("Number of decryption shares needed (w)");
		JLabel typeLabel = new JLabel("Type of election");
		JPanel nPanel = new JPanel();
		JPanel MPanel = new JPanel();
		JPanel kPanel = new JPanel();
		JPanel tPanel = new JPanel();
		JPanel LPanel = new JPanel();
		JPanel lPanel = new JPanel();
		JPanel wPanel = new JPanel();
		JPanel typePanel = new JPanel();

		JPanel nLabelPanel = new JPanel();
		JPanel MLabelPanel = new JPanel();
		JPanel kLabelPanel = new JPanel();
		JPanel tLabelPanel = new JPanel();
		JPanel LLabelPanel = new JPanel();
		JPanel lLabelPanel = new JPanel();
		JPanel wLabelPanel = new JPanel();
		JPanel typeLabelPanel = new JPanel();

		nLabelPanel.add(nLabel);
		MLabelPanel.add(MLabel);
		kLabelPanel.add(kLabel);
		tLabelPanel.add(tLabel);
		LLabelPanel.add(LLabel);
		lLabelPanel.add(lLabel);
		wLabelPanel.add(wLabel);
		typeLabelPanel.add(typeLabel);

		nSpin = new JSpinner();
		nSpin.setValue(new Integer(1024));
		((JSpinner.NumberEditor) nSpin.getEditor()).getTextField().setColumns(
				17);
		nPanel.add(nSpin);
		MSpin = new JSpinner();
		MSpin.setValue(new Integer(1000));
		((JSpinner.NumberEditor) MSpin.getEditor()).getTextField().setColumns(
				17);
		MPanel.add(MSpin);
		kSpin = new JSpinner();
		kSpin.setValue(new Integer(2));
		((JSpinner.NumberEditor) kSpin.getEditor()).getTextField().setColumns(
				17);
		kPanel.add(kSpin);
		tSpin = new JSpinner();
		tSpin.setValue(new Integer(80));
		((JSpinner.NumberEditor) tSpin.getEditor()).getTextField().setColumns(
				17);
		tPanel.add(tSpin);
		LSpin = new JSpinner();
		LSpin.setValue(new Integer(6));
		((JSpinner.NumberEditor) LSpin.getEditor()).getTextField().setColumns(
				17);
		LPanel.add(LSpin);
		lSpin = new JSpinner();
		lSpin.setValue(new Integer(10));
		((JSpinner.NumberEditor) lSpin.getEditor()).getTextField().setColumns(
				17);
		lPanel.add(lSpin);
		wSpin = new JSpinner();
		wSpin.setValue(new Integer(5));
		((JSpinner.NumberEditor) wSpin.getEditor()).getTextField().setColumns(
				17);
		wPanel.add(wSpin);
		String[] s = new String[4];
		s[0] = "1-out-of-L";
		s[1] = "1-out-of-L with Binary Coded Votes";
		s[2] = "k-out-of-L";
		s[3] = "k-out-of-L with Binary Coded Votes";

		typeBox = new JComboBox(s);
		typePanel.add(typeBox);
		pageOne.add(nLabelPanel);
		pageOne.add(nPanel);
		pageOne.add(MLabelPanel);
		pageOne.add(MPanel);
		pageOne.add(kLabelPanel);
		pageOne.add(kPanel);
		pageOne.add(tLabelPanel);
		pageOne.add(tPanel);
		pageOne.add(LLabelPanel);
		pageOne.add(LPanel);
		pageOne.add(lLabelPanel);
		pageOne.add(lPanel);
		pageOne.add(wLabelPanel);
		pageOne.add(wPanel);
		pageOne.add(typeLabelPanel);
		pageOne.add(typePanel);

		JButton pageOneButton = new JButton("Next Page");
		pageOneButton.addActionListener(this);
		bottomLayer.add(pageOneButton, BorderLayout.SOUTH);
		bottomLayer.validate();

	}

	private void loadParametersPageOne() {
		keylength = ((Integer) nSpin.getValue()).intValue();
		M = ((Integer) MSpin.getValue()).intValue();
		k = ((Integer) kSpin.getValue()).intValue();
		L = ((Integer) LSpin.getValue()).intValue();
		l = ((Integer) lSpin.getValue()).intValue();
		w = ((Integer) wSpin.getValue()).intValue();
		String typeOfElection = (String) typeBox.getSelectedItem();
		if (typeOfElection.equals("1-out-of-L")) {
			type = 0;
			k = 1;
		} else if (typeOfElection.equals("k-out-of-L"))
			type = 1;
		else if (typeOfElection.equals("1-out-of-L with Binary Coded Votes")) {
			type = 2;
			k = 1;
		}

		else if (typeOfElection.equals("k-out-of-L with Binary Coded Votes"))
			type = 3;

		t = ((Integer) tSpin.getValue()).intValue();
		try {
			rec = new RemoteEntityConnectionEA(port);
		} catch (RemoteException e) {
			showDialog("Could not create RMI Registry. Port may be busy");
		}

	}

	private void createPageTwo() {
		JLabel pageTwoLabel = new JLabel("Set up the election (2/2)");
		JSplitPane pageTwoPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		JScrollPane weightsScroll = new JScrollPane();
		JPanel weightsPanel = new JPanel();
		weightsScroll.add(weightsPanel);
		weightsPanel.setLayout(new GridLayout(k, 2));

		JScrollPane nameScroll = new JScrollPane();
		JPanel namePanel = new JPanel();
		nameScroll.add(namePanel);
		namePanel.setLayout(new GridLayout(L, 2));

		bottomLayer.removeAll();
		bottomLayer.add(pageTwoLabel, BorderLayout.NORTH);
		bottomLayer.add(pageTwoPanel);
		pageTwoPanel.setTopComponent(weightsPanel);
		pageTwoPanel.setBottomComponent(namePanel);

		weights = new JLabel[k];
		weightsSpin = new JSpinner[k];
		if (k > 1) {
			for (int i = 0; i < k; i++) {
				weights[i] = new JLabel("Weight for Vote " + (i + 1));
				weightsSpin[i] = new JSpinner();
				weightsSpin[i].setValue(new Integer(1));
				((JSpinner.NumberEditor) weightsSpin[i].getEditor())
						.getTextField().setColumns(17);
				JPanel temp = new JPanel();
				JPanel temp2 = new JPanel();
				temp.add(weights[i]);
				temp2.add(weightsSpin[i]);
				weightsPanel.add(temp);
				weightsPanel.add(temp2);
			}
		}

		JLabel[] nameLabels = new JLabel[L];
		nameFields = new JTextField[L];
		for (int i = 0; i < L; i++) {
			nameLabels[i] = new JLabel("Name for Candidate " + (i + 1));
			nameFields[i] = new JTextField();
			nameFields[i].setColumns(25);
			JPanel temp3 = new JPanel();
			JPanel temp4 = new JPanel();
			temp3.add(nameLabels[i]);
			temp4.add(nameFields[i]);
			namePanel.add(temp3);
			namePanel.add(temp4);
		}
		JButton pageTwoButton = new JButton("Start Election");
		pageTwoButton.addActionListener(this);
		bottomLayer.add(pageTwoButton, BorderLayout.SOUTH);

		bottomLayer.revalidate();

	}

	private void loadParametersPageTwo() {
		boolean good2go = true;

		for (int i = 0; i < L; i++) {
			System.out.println(nameFields[i].getText());
			if (nameFields[i].getText().equals("")) {
				good2go = false;
				break;
			}
		}
		if (!good2go)
			showDialog("Fill out all names!");
		else {
			showProgBar("Generating Keys");
			voteNames = new String[L];
			for (int i = 0; i < L; i++)
				voteNames[i] = nameFields[i].getText();

			voteWeights = new int[k];
			if (k > 1) {
				for (int i = 0; i < k; i++)
					voteWeights[i] = ((Integer) weightsSpin[i].getValue())
							.intValue();
			} else {
				voteWeights[0] = 1;
			}
			this.generateKeys();

		}
	}

	private void createPageThree() {
		bottomLayer.removeAll();
		JLabel pageThreeText = new JLabel("Election in process");
		JPanel panel = new JPanel(new GridLayout(3, 2));
		JLabel noServersLabel = new JLabel("Number of servers acquired key");
		noServers = new JLabel("0");
		JLabel noVotesLabel = new JLabel("Number of voters connected");
		noVotes = new JLabel("0");
		JLabel noVotesCastLabel = new JLabel("Number of votes cast");
		noVotesCast = new JLabel("0");

		JPanel noServersLabelPanel = new JPanel();
		JPanel noServersPanel = new JPanel();
		JPanel noVotesLabelPanel = new JPanel();
		JPanel noVotesPanel = new JPanel();
		JPanel noVotesCastLabelPanel = new JPanel();
		JPanel noVotesCastPanel = new JPanel();

		noServersLabelPanel.add(noServersLabel);
		noServersPanel.add(noServers);
		noVotesLabelPanel.add(noVotesLabel);
		noVotesPanel.add(noVotes);
		noVotesCastLabelPanel.add(noVotesCastLabel);
		noVotesCastPanel.add(noVotesCast);

		panel.add(noServersLabelPanel);
		panel.add(noServersPanel);
		panel.add(noVotesLabelPanel);
		panel.add(noVotesPanel);
		panel.add(noVotesCastLabelPanel);
		panel.add(noVotesCastPanel);

		bottomLayer.add(panel);
		bottomLayer.add(pageThreeText, BorderLayout.NORTH);

		JPanel buttonLayer = new JPanel();
		JButton pageThreeButton = new JButton("End Election");
		pageThreeButton.addActionListener(this);

		JButton refreshButton = new JButton("Refresh");
		refreshButton.addActionListener(this);

		buttonLayer.add(refreshButton);
		buttonLayer.add(pageThreeButton);
		bottomLayer.add(buttonLayer, BorderLayout.SOUTH);

		bottomLayer.validate();
	}

	private void createPageFour() {
		bottomLayer.removeAll();
		JLabel pageFourText = new JLabel(
				"Election finished. Waiting for decryption servers to calculate shares");
		bottomLayer.add(pageFourText, BorderLayout.NORTH);
		JPanel panel = new JPanel(new GridLayout(2, 2));

		JPanel noOfSharesLabelPanel = new JPanel();
		JPanel noOfSharesPanel = new JPanel();
		JLabel noOfSharesLabel = new JLabel("Number of Shares acquired");
		noOfShares = new JLabel("0");
		noOfSharesLabelPanel.add(noOfSharesLabel);
		noOfSharesPanel.add(noOfShares);

		JPanel requiredLabelPanel = new JPanel();
		JPanel requiredPanel = new JPanel();
		JLabel requiredLabel = new JLabel("Number of VALID shares required");
		JLabel required = new JLabel(ep.getW() + "");
		requiredLabelPanel.add(requiredLabel);
		requiredPanel.add(required);

		panel.add(noOfSharesLabelPanel);
		panel.add(noOfSharesPanel);
		panel.add(requiredLabelPanel);
		panel.add(requiredPanel);

		JPanel buttonLayer = new JPanel();
		JButton refresh = new JButton("Refresh");
		refresh.setActionCommand("Refresh2");
		refresh.addActionListener(this);

		JButton calculate = new JButton("Calculate Results");
		calculate.addActionListener(this);
		buttonLayer.add(refresh);
		buttonLayer.add(calculate);

		bottomLayer.add(panel);
		bottomLayer.add(buttonLayer, BorderLayout.SOUTH);
		bottomLayer.validate();

	}

	private void showResultsPage(BigInteger[] results) {
		bottomLayer.removeAll();
		JLabel resultsLabel = new JLabel("Results of the election");
		bottomLayer.add(resultsLabel, BorderLayout.NORTH);
		JPanel resultsPanel = new JPanel(new GridLayout(results.length, 2));
		JLabel[] nameList = new JLabel[results.length];
		JLabel[] resultsList = new JLabel[results.length];
		for (int i = 0; i < results.length; i++) {
			JPanel temp = new JPanel();
			JPanel temp2 = new JPanel();
			nameList[i] = new JLabel(ep.getVoteNames()[i]);
			resultsList[i] = new JLabel(results[i].toString());
			temp.add(nameList[i]);
			temp2.add(resultsList[i]);
			resultsPanel.add(temp);
			resultsPanel.add(temp2);

		}
		bottomLayer.add(resultsPanel);
		bottomLayer.validate();

	}

	private void calculateResults() {
		System.out.println("In CalculateResults");
		BigInteger results[] = null;
		ea = new ElectionAuthority(kg.getPk(), ep, bb, false);
		try {
			results = ea.combineShares(ep.getElectionType());
			showResultsPage(results);
		} catch (RemoteException e) {
			showDialog("Could not connect to BulletinBoard");
		} catch (InsufficientSharesException e) {
			showDialog("Not enough VALID shares acquired");
		}

	}

	public void actionPerformed(ActionEvent e) {
		System.out.println(e.getActionCommand());
		if (e.getActionCommand().equals("Next Page")) {
			loadParametersPageOne();
			createPageTwo();
		} else if (e.getActionCommand().equals("Start Election")) {
			loadParametersPageTwo();
		} else if (e.getActionCommand().equals("End Election")) {
			System.out.println("End Election!!");

			try {
				if (bb == null) {
					bb = rec.getBulletinBoard();
				}
				bb.endElection();
				createPageFour();
			} catch (AccessException e1) {
				showDialog("No BulletinBoard found. Access Denied");
			} catch (RemoteException e1) {
				showDialog("No BulletinBoard found");
			} catch (NotBoundException e1) {
				showDialog("No BulletinBoard found");
			}

		} else if (e.getActionCommand().equals("Refresh")) {
			System.out.println("Refresh");

			try {
				noServers.setText(es.getNoOfDecryptionServers() + "");
				noVotes.setText(es.getNoOfVoters() + "");
			} catch (RemoteException e2) {
				showDialog("Refresh Unsuccessful.");
			}

			try {
				if (bb == null) {
					bb = rec.getBulletinBoard();
				}
				noVotesCast.setText(bb.getNoOfVotes() + "");

			} catch (AccessException e1) {
				showDialog("No BulletinBoard found. Access Denied");
			} catch (RemoteException e1) {
				showDialog("No BulletinBoard found");
			} catch (NotBoundException e1) {
				showDialog("No BulletinBoard found");
			}

		} else if (e.getActionCommand().equals("Refresh2")) {
			try {
				if (bb == null) {
					bb = rec.getBulletinBoard();
				}
				noOfShares.setText(bb.getNoOfShares() + "");

			} catch (AccessException e1) {
				showDialog("No BulletinBoard found. Access Denied");
			} catch (RemoteException e1) {
				showDialog("No BulletinBoard found");
			} catch (NotBoundException e1) {
				showDialog("No BulletinBoard found");
			}
		} else if (e.getActionCommand().equals("Calculate Results")) {
			calculateResults();

		}

	}

	private void generateKeys() {
		new Thread(this).start();
	}

	private void showProgBar(String msg) {
		this.msg = msg;
		progWin = new JWindow();
		// progWin.setAlwaysOnTop(true);
		progWin.setFocusable(false);
		JLabel messageLabel = new JLabel(msg);
		JPanel panel = new JPanel(new BorderLayout());
		JProgressBar progBar = new JProgressBar();
		progBar.setIndeterminate(true);

		progWin.setSize(200, 100);
		progWin.setVisible(true);
		progWin.setLocation(this.getX() + this.getWidth() / 2 - 40, this.getY()
				+ this.getHeight() / 2);
		progWin.getContentPane().add(panel);
		panel.add(messageLabel, BorderLayout.NORTH);
		panel.add(progBar);
		progWin.validate();

	}

	private void closeProgBar() {
		progWin.dispose();
	}

	private void showDialog(String msg) {
		JOptionPane.showMessageDialog(this, msg);

	}

	public void run() {
		// Key Generation
		rand = new SecureRandom();
		ep = new ElectionProperties(keylength, L, M, l, w, t, k, voteNames,
				type, voteWeights);
		kg = new KeyGenerator(rand, keylength, ep.getS());
		System.out.println("n: " + kg.getN());
		ks = kg.generateKeys(ep.getl(), ep.getW());
		
		try {
			es = rec.bindElectionServer(ep, kg.getPk(), ks);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		createPageThree();
		System.out.println("Keys Generated");
		closeProgBar();
	}
/** Starts the election authority
 * 
 * @param args
 */
	public static void main(String[] args) {

		int port = 2004;
		if(args.length == 0){
			System.out.println("");
			System.out.println("RMI Registry running on default port 2004");
		}
		else{
		try {
			 port = Integer.parseInt(args[0]);
		} catch (NumberFormatException e) {
			System.out.println("");
			System.out.println("Run as: java ea.GUI xxxx");
			System.out.println("");
			System.out.println("Where xxxx = portnumber for RMI registry");
			System.out.println("");
			System.exit(1);
		}
		System.out.println("");
		System.out.println("RMI Registry running on port "+port);
		}
			
		new GUI(port);
	}

}
