package ea;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import remoteInterfaces.BulletinBoard;
import remoteInterfaces.ElectionServer;
import votingSystem.ElectionProperties;
import votingSystem.KeyShare;
import votingSystem.PublicKey;
/** The class responsible for starting the RMI Registry as well as handling interaction with the registry.
 * 
 * @author bent
 *
 */
public class RemoteEntityConnectionEA {
	Registry registry;
/** Starts a new RMI registry at the given port.
 * 
 * @param port
 * @throws RemoteException
 */
	public RemoteEntityConnectionEA(int port) throws RemoteException {
		LocateRegistry.createRegistry(port);
		registry = LocateRegistry.getRegistry(port);

	}
/** Returns a stub to the remote object BulletinBoard
 * 
 * @return The stub of the BulletinBoard
 * @throws AccessException
 * @throws RemoteException
 * @throws NotBoundException
 */
	public BulletinBoard getBulletinBoard() throws AccessException,
			RemoteException, NotBoundException {
		BulletinBoard obj;

		obj = (BulletinBoard) registry.lookup("BulletinBoard");

		return obj;
	}
/** Binds the ElectionServer to the registry.
 * 
 * @param ep The parameters for the election.
 * @param pk The public key for the election.
 * @param ks The array of KeyShares for the decryption servers. 
 * @return The stub of the ElectionServer
 * @throws RemoteException
 */
	public ElectionServer bindElectionServer(ElectionProperties ep,
			PublicKey pk, KeyShare[] ks) throws RemoteException {
		ElectionServer stub = null;
		ElectionServer_Impl esi = new ElectionServer_Impl(ep, pk, ks);

		stub = (ElectionServer) UnicastRemoteObject.exportObject(esi, 0);
		System.out.println("ElectionServer not yet bound in registry");
		registry.rebind("ElectionServer", stub);
		System.out.println("ElectionServer bound in registry");
		return stub;
	}

}
