package remoteInterfaces;

import java.util.ArrayList;

import votingSystem.BinaryVote;
import votingSystem.DecryptedShare;
import votingSystem.MultipleBinaryVotes;
import votingSystem.MultipleVotes;
import votingSystem.Vote;

/** The interface needed for using the remote object BulletinBoard bound in the RMI registry
 * 
 * @author bent
 *
 */
public interface BulletinBoard extends java.rmi.Remote {

	/**
	 * Posts a Vote to the bulletin board
	 * 
	 * @param v
	 *            The vote
	 * 
	 */
	public void postVote(Vote v) throws java.rmi.RemoteException;
	/**
	 * Posts a BinaryVote to the bulletin board
	 * 
	 * @param v
	 *            The BinaryVote
	 * 
	 */
	public void postVote(BinaryVote v) throws java.rmi.RemoteException;
	/**
	 * Posts MultipleVotes to the bulletin board
	 * 
	 * @param v
	 *            The votes
	 * 
	 */
	public void postVote(MultipleVotes v) throws java.rmi.RemoteException;
	/**
	 * Posts MultipleBinaryVotes to the bulletin board
	 * 
	 * @param v
	 *            The votes
	 * 
	 */
	public void postVote(MultipleBinaryVotes v) throws java.rmi.RemoteException;
	/**
	 * Gets the Votes in the case of a single candidate election
	 * 
	 * @return The Votes in an array
	 */
	public Vote[] getVotes() throws java.rmi.RemoteException;
	/**
	 * Gets the BinaryVotes in the case of a single candidate election
	 * 
	 * @return The BinaryVotes in an array
	 */
	public BinaryVote[] getBinaryVotes() throws java.rmi.RemoteException;
	/**
	 * Gets the MultipleVotes in the case of a multi candidate election
	 * 
	 * @return The MultipleVotes in an array
	 */
	public MultipleVotes[] getMultipleVotes() throws java.rmi.RemoteException;
	/**
	 * Gets the MultipleBinaryVotes in the case of a multi candidate election
	 * 
	 * @return The MultipleBinaryVotes in an array
	 */
	public MultipleBinaryVotes[] getMultipleBinaryVotes()
			throws java.rmi.RemoteException;
	/**
	 * Gets the number of votes cast in the election
	 * 
	 * @return The number of votes
	 * 
	 */
	public int getNoOfVotes() throws java.rmi.RemoteException;
	/**
	 * Ends the election, thus disallowing the posting of additional votes
	 * 
	 */
	public void endElection() throws java.rmi.RemoteException;
	/**
	 * Gets the number of DecryptedShares posted by the DecryptionServers
	 * 
	 * @return The number of DecryptedShares
	 * 
	 */
	public int getNoOfShares() throws java.rmi.RemoteException;
	/**
	 * Posts a DecryptedShare to the bulletin board
	 * 
	 * @param ds
	 *            The DecryptedShare
	 */
	public void postShare(DecryptedShare ds) throws java.rmi.RemoteException;
	/**
	 * Gets the status of the election
	 * 
	 * @return The status of the election. True if it is finished, false if not.
	 * 
	 */
	public boolean getElectionStatus() throws java.rmi.RemoteException;
	/**
	 * Posts multiple DecryptedShares to the bulletin board in a multiple vote
	 * election
	 * 
	 * @param ds
	 *            The DecryptedShares
	 */
	public void postShare(DecryptedShare[] ds) throws java.rmi.RemoteException;
	/**
	 * Gets the DecryptedShares in the case of a multiple vote election
	 * 
	 * @return The shares as DecryptedShare-arrays in an ArrayList
	 * 
	 */
	public ArrayList getKOfLShares() throws java.rmi.RemoteException;
	/**
	 * Gets the DecryptedShares in the case of a single vote election
	 * 
	 * @return The shares in an array
	 * 
	 */
	public DecryptedShare[] getOneOfLShares() throws java.rmi.RemoteException;
}
