package simulator;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.security.SecureRandom;
import java.util.*;
import javax.swing.*;
import voter.*;
import votingSystem.*;
import ds.*;
import ea.RemoteEntityConnectionEA;
import exceptions.InsufficientSharesException;
import exceptions.NoValidVotesException;
import remoteInterfaces.*;
import bb.*;
import ea.*;


public class Simulator extends JFrame implements ActionListener, Runnable{
	
	// Parameters for the Election
	private int keylength, t, M, L, l, w, type, k, noOfVoters;
	
	private JPanel bottomLayer;
	
//	 Boxes for pageOne
	private JSpinner nSpin;

	private JSpinner MSpin;

	private JSpinner kSpin;

	private JSpinner tSpin;

	private JSpinner LSpin;

	private JSpinner lSpin;

	private JSpinner wSpin;

	private JSpinner noOfVotersSpin;
	
	private JComboBox typeBox;
	
	private String msg = "";
	
	private JWindow progWin;
	
	private JLabel messageLabel;
	
	//Simulation results
	
	private long keygenTime, combineTime, totalTime;
	private long[] voterTime, decryptTime;
	
	/**
	 * Starts a new simulator
	 *
	 */
		public Simulator() {
			super("Election Simulator");
			try {
				UIManager
						.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			} catch (Exception e) {
			}

			bottomLayer = new JPanel(new BorderLayout());

			this.getContentPane().add(bottomLayer);
			this.setSize(800, 600);
			this.setVisible(true);
			this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			createPageOne();
		}
		
		private void createPageOne() {
			JLabel pageOneText = new JLabel("Set up the election (1/2)");
			JPanel pageOne = new JPanel(new GridLayout(9, 2));
			bottomLayer.add(pageOneText, BorderLayout.NORTH);
			bottomLayer.add(pageOne);
			JLabel nLabel = new JLabel("Keylength of Public Key");
			JLabel MLabel = new JLabel("Maximum number of voters (M)");
			JLabel kLabel = new JLabel("Number of votes (t)");
			JLabel tLabel = new JLabel(
					"Security parameter. Length of bitchallenges (q)");
			JLabel LLabel = new JLabel("Number of candidates (L)");
			JLabel lLabel = new JLabel("Number of decryption servers (l)");
			JLabel wLabel = new JLabel("Number of decryption shares needed (w)");
			JLabel typeLabel = new JLabel("Type of election");
			JLabel noOfVotersLabel = new JLabel("Number of voters used in the simulation");
			JPanel nPanel = new JPanel();
			JPanel MPanel = new JPanel();
			JPanel kPanel = new JPanel();
			JPanel tPanel = new JPanel();
			JPanel LPanel = new JPanel();
			JPanel lPanel = new JPanel();
			JPanel wPanel = new JPanel();
			JPanel typePanel = new JPanel();
			JPanel noOfVotersPanel = new JPanel();
			
			JPanel nLabelPanel = new JPanel();
			JPanel MLabelPanel = new JPanel();
			JPanel kLabelPanel = new JPanel();
			JPanel tLabelPanel = new JPanel();
			JPanel LLabelPanel = new JPanel();
			JPanel lLabelPanel = new JPanel();
			JPanel wLabelPanel = new JPanel();
			JPanel typeLabelPanel = new JPanel();
			JPanel noOfVotersLabelPanel = new JPanel();

			nLabelPanel.add(nLabel);
			MLabelPanel.add(MLabel);
			kLabelPanel.add(kLabel);
			tLabelPanel.add(tLabel);
			LLabelPanel.add(LLabel);
			lLabelPanel.add(lLabel);
			wLabelPanel.add(wLabel);
			typeLabelPanel.add(typeLabel);
			noOfVotersLabelPanel.add(noOfVotersLabel);
			
			nSpin = new JSpinner();
			nSpin.setValue(new Integer(1024));
			((JSpinner.NumberEditor) nSpin.getEditor()).getTextField().setColumns(
					17);
			nPanel.add(nSpin);
			MSpin = new JSpinner();
			MSpin.setValue(new Integer(1000));
			((JSpinner.NumberEditor) MSpin.getEditor()).getTextField().setColumns(
					17);
			MPanel.add(MSpin);
			kSpin = new JSpinner();
			kSpin.setValue(new Integer(2));
			((JSpinner.NumberEditor) kSpin.getEditor()).getTextField().setColumns(
					17);
			kPanel.add(kSpin);
			tSpin = new JSpinner();
			tSpin.setValue(new Integer(80));
			((JSpinner.NumberEditor) tSpin.getEditor()).getTextField().setColumns(
					17);
			tPanel.add(tSpin);
			LSpin = new JSpinner();
			LSpin.setValue(new Integer(6));
			((JSpinner.NumberEditor) LSpin.getEditor()).getTextField().setColumns(
					17);
			LPanel.add(LSpin);
			lSpin = new JSpinner();
			lSpin.setValue(new Integer(10));
			((JSpinner.NumberEditor) lSpin.getEditor()).getTextField().setColumns(
					17);
			lPanel.add(lSpin);
			wSpin = new JSpinner();
			wSpin.setValue(new Integer(5));
			((JSpinner.NumberEditor) wSpin.getEditor()).getTextField().setColumns(
					17);
			wPanel.add(wSpin);
			String[] s = new String[4];
			s[0] = "1-out-of-L";
			s[1] = "1-out-of-L with Binary Coded Votes";
			s[2] = "k-out-of-L";
			s[3] = "k-out-of-L with Binary Coded Votes";

			typeBox = new JComboBox(s);
			typePanel.add(typeBox);
			
			noOfVotersSpin = new JSpinner();
			noOfVotersSpin.setValue(new Integer(5));
			((JSpinner.NumberEditor) noOfVotersSpin.getEditor()).getTextField().setColumns(
					17);
			noOfVotersPanel.add(noOfVotersSpin);
		
			
			pageOne.add(nLabelPanel);
			pageOne.add(nPanel);
			pageOne.add(MLabelPanel);
			pageOne.add(MPanel);
			pageOne.add(kLabelPanel);
			pageOne.add(kPanel);
			pageOne.add(tLabelPanel);
			pageOne.add(tPanel);
			pageOne.add(LLabelPanel);
			pageOne.add(LPanel);
			pageOne.add(lLabelPanel);
			pageOne.add(lPanel);
			pageOne.add(wLabelPanel);
			pageOne.add(wPanel);
			pageOne.add(typeLabelPanel);
			pageOne.add(typePanel);
			pageOne.add(noOfVotersLabelPanel);
			pageOne.add(noOfVotersPanel);

			JButton pageOneButton = new JButton("Start Simulation");
			pageOneButton.addActionListener(this);
			bottomLayer.add(pageOneButton, BorderLayout.SOUTH);
			bottomLayer.validate();

		}
		
		private void loadParametersPageOne() {
			keylength = ((Integer) nSpin.getValue()).intValue();
			M = ((Integer) MSpin.getValue()).intValue();
			k = ((Integer) kSpin.getValue()).intValue();
			L = ((Integer) LSpin.getValue()).intValue();
			l = ((Integer) lSpin.getValue()).intValue();
			w = ((Integer) wSpin.getValue()).intValue();
			String typeOfElection = (String) typeBox.getSelectedItem();
			if (typeOfElection.equals("1-out-of-L")) {
				type = 0;
				k = 1;
			} else if (typeOfElection.equals("k-out-of-L"))
				type = 1;
			else if (typeOfElection.equals("1-out-of-L with Binary Coded Votes")) {
				type = 2;
				k = 1;
			}

			else if (typeOfElection.equals("k-out-of-L with Binary Coded Votes"))
				type = 3;

			t = ((Integer) tSpin.getValue()).intValue();
			noOfVoters = ((Integer)noOfVotersSpin.getValue()).intValue();

		}
		private void showResults(){
			bottomLayer.removeAll();
			JPanel resultsPage = new JPanel(new GridLayout(8,2));
			
			JLabel totalLabel = new JLabel("Total time:");
			JLabel keygenLabel = new JLabel("Key generation time: ");
			JLabel meanVoterLabel = new JLabel("Mean vote encryption time: ");
			JLabel totalVoterLabel = new JLabel("Total vote encryption time: ");
			JLabel meanDecryptLabel = new JLabel("Mean share decryption time: ");
			JLabel meanDVLabel = new JLabel("Mean share decryption time per vote: ");
			JLabel totalDecryptLabel = new JLabel("Total share decryption time: ");
			JLabel combiLabel = new JLabel("Share combination time: ");
			
			JPanel totalLabelPanel = new JPanel();
			JPanel keygenLabelPanel = new JPanel();
			JPanel meanVoterLabelPanel = new JPanel();
			JPanel totalVoterLabelPanel = new JPanel();
			JPanel meanDecryptLabelPanel = new JPanel();
			JPanel meanDVLabelPanel = new JPanel();
			JPanel totalDecryptLabelPanel = new JPanel();
			JPanel combiLabelPanel = new JPanel(); 
			
			totalLabelPanel.add(totalLabel);
			keygenLabelPanel.add(keygenLabel);
			meanVoterLabelPanel.add(meanVoterLabel);
			totalVoterLabelPanel.add(totalVoterLabel);
			meanDecryptLabelPanel.add(meanDecryptLabel);
			meanDVLabelPanel.add(meanDVLabel);
			totalDecryptLabelPanel.add(totalDecryptLabel);
			combiLabelPanel.add(combiLabel);
			
			
			long totalVoterTime = 0;
			for(int i = 0;i<voterTime.length;i++)
				totalVoterTime += voterTime[i];
			
			long totalDecryptTime = 0;
			for(int i=0;i<decryptTime.length;i++)
				totalDecryptTime+= decryptTime[i];
			
			JLabel total = new JLabel(totalTime+" ms.");
			JLabel keygen = new JLabel(keygenTime+" ms.");
			JLabel meanVoter = new JLabel((totalVoterTime/voterTime.length) + " ms.");
			JLabel totalVoter = new JLabel(totalVoterTime+ " ms.");
			JLabel meanDecrypt = new JLabel((totalDecryptTime/decryptTime.length)+ " ms.");
			JLabel meanDV = new JLabel((totalDecryptTime/(decryptTime.length*voterTime.length))+ " ms.");
			JLabel totalDecrypt = new JLabel(totalDecryptTime+" ms.");
			JLabel combi = new JLabel(combineTime + " ms.");
			
			JPanel totalPanel = new JPanel();
			JPanel keygenPanel = new JPanel();
			JPanel meanVoterPanel = new JPanel();
			JPanel totalVoterPanel = new JPanel();
			JPanel meanDecryptPanel = new JPanel();
			JPanel meanDVPanel = new JPanel();
			JPanel totalDecryptPanel = new JPanel();
			JPanel combiPanel = new JPanel();
			
			totalPanel.add(total);
			keygenPanel.add(keygen);
			meanVoterPanel.add(meanVoter);
			totalVoterPanel.add(totalVoter);
			meanDecryptPanel.add(meanDecrypt);
			meanDVPanel.add(meanDV);
			totalDecryptPanel.add(totalDecrypt);
			combiPanel.add(combi);
			
			resultsPage.add(totalLabelPanel);
			resultsPage.add(totalPanel);
			resultsPage.add(keygenLabelPanel);
			resultsPage.add(keygenPanel);
			resultsPage.add(meanVoterLabelPanel);
			resultsPage.add(meanVoterPanel);
			resultsPage.add(totalVoterLabelPanel);
			resultsPage.add(totalVoterPanel);
			resultsPage.add(meanDecryptLabelPanel);
			resultsPage.add(meanDecryptPanel);
			resultsPage.add(meanDVLabelPanel);
			resultsPage.add(meanDVPanel);
			resultsPage.add(totalDecryptLabelPanel);
			resultsPage.add(totalDecryptPanel);
			resultsPage.add(combiLabelPanel);
			resultsPage.add(combiPanel);
			
			bottomLayer.add(resultsPage);
			bottomLayer.validate();

			
		}
		private void startSimulation(){
			new Thread(this).start();
			this.showProgBar("Starting Simulation");
		}
		
		public void run(){
			
			long time1, time2, time1Total;
			time1Total = System.currentTimeMillis();
			Random rand = new SecureRandom();
			int[] values = new int[k];
			for(int i=0;i<k;i++){
				values[i] = 1;
			}
			String names[] = new String[L];
			for(int i= 0;i<L;i++)
				names[i] = "Hest"+i;
			ElectionProperties ep = new ElectionProperties(keylength, L, M, l, w, t, k, names,
					type, values);
			//Generate Keys
			time1 = System.currentTimeMillis();
			KeyGenerator kg = new KeyGenerator(rand, keylength, ep.getS());
			KeyShare[] ks = kg.generateKeys(ep.getl(), ep.getW());
			time2 = System.currentTimeMillis();
			keygenTime = time2-time1;
			
			PublicKey pk = kg.getPk();
			
			messageLabel.setText("Started encrypting votes");
			voterTime = new long[noOfVoters];
			
		
			BulletinBoard bb = new BulletinBoard_Impl();
			
			
			switch(type){
			
			case 0:
				for(int i=0; i<noOfVoters; i++){
					time1 = System.currentTimeMillis();
					Voter v = new Voter(i, ep, pk, rand, null);
					try{
					bb.postVote(v.encryptVote((int)(Math.random()*L)));
					}
					catch(Exception e){e.printStackTrace();}
					time2 = System.currentTimeMillis();
					voterTime[i] = time2-time1;
				}
				break;
			
			case 1: 
				int noOfVotes = k;
				int[] candidates = new int[noOfVotes];
				for(int i=0;i<noOfVotes;i++){
					candidates[i] = i;
				}
				for(int i=0; i<noOfVoters; i++){
					time1 = System.currentTimeMillis();
					Voter v = new Voter(i, ep, pk, rand, null);
					try{
					bb.postVote(v.encryptMultipleVotes(candidates));
					}
					catch(Exception e){e.printStackTrace();}
					time2 = System.currentTimeMillis();
					voterTime[i] = time2-time1;
				}
				break;
			
			case 2: 
				for(int i=0; i<noOfVoters; i++){
					time1 = System.currentTimeMillis();
					Voter v = new Voter(i, ep, pk, rand, null);
					try{
					bb.postVote(v.encryptVoteBinary((int)(Math.random()*L)));
					}
					catch(Exception e){e.printStackTrace();}
					time2 = System.currentTimeMillis();
					voterTime[i] = time2-time1;
				}
				break;
			
			case 3: 	
				int noOfVotes2 = k;
			int[] bincandidates = new int[noOfVotes2];
			for(int i=0;i<noOfVotes2;i++){
				bincandidates[i] = i;
			}
			for(int i=0; i<noOfVoters; i++){
				time1 = System.currentTimeMillis();
				Voter v = new Voter(i, ep, pk, rand, null);
				try{
				bb.postVote(v.encryptMultipleVoteBinary(bincandidates));
				}
				catch(Exception e){e.printStackTrace();}
				time2 = System.currentTimeMillis();
				voterTime[i] = time2-time1;
			}
			break;
				
			}
			decryptTime = new long[l];
			Server[] dsList = new Server[l];
			for(int i=0;i<dsList.length; i++){
				dsList[i] = new Server(i, ks[i], ep, rand, pk,bb);
			}
			
			messageLabel.setText("Starting decryption of votes");
		
			for(int i=0;i<l; i++){
				time1 = System.currentTimeMillis();
				try {
					dsList[i].postShare();
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (NoValidVotesException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				time2 = System.currentTimeMillis();
				decryptTime[i] = time2-time1;
			}
			
			messageLabel.setText("Starting combination of shares");
			ElectionAuthority ea = new ElectionAuthority(pk, ep, bb, false);
			
			time1 = System.currentTimeMillis();
			try {
				ea.combineShares(type);
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InsufficientSharesException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			time2 = System.currentTimeMillis();
			combineTime = time2-time1;
			totalTime = time2 - time1Total;
			closeProgBar();
			showResults();
		}
		
		public void actionPerformed(ActionEvent e){
			if (e.getActionCommand().equals("Start Simulation")) {
				loadParametersPageOne();
				startSimulation();
			}
		}
		private void showProgBar(String msg) {
			this.msg = msg;
			progWin = new JWindow();
			// progWin.setAlwaysOnTop(true);
			progWin.setFocusable(false);
			 messageLabel = new JLabel(msg);
			JPanel panel = new JPanel(new BorderLayout());
			JProgressBar progBar = new JProgressBar();
			progBar.setIndeterminate(true);
			
			progWin.setSize(300, 100);
			progWin.setVisible(true);
			progWin.setLocation(this.getX() + this.getWidth() / 2 - 40, this.getY()
					+ this.getHeight() / 2);
			progWin.getContentPane().add(panel);
			panel.add(messageLabel, BorderLayout.NORTH);
			panel.add(progBar);
			progWin.validate();

		}

		private void closeProgBar() {
			progWin.dispose();
		}

		private void showDialog(String msg) {
			JOptionPane.showMessageDialog(this, msg);

		}
		public static void main(String[] args){
			new Simulator();
		}
}
