package voter;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.security.SecureRandom;
import java.util.Random;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.UIManager;

import remoteInterfaces.BulletinBoard;
import remoteInterfaces.ElectionServer;
import votingSystem.ElectionProperties;
import votingSystem.PublicKey;
import exceptions.IncorrectVoteException;
/** The class that starts the voter application. This class mainly controls the user interface.
 * 
 * @author bent
 *
 */
public class GUI extends JFrame implements ActionListener {

	private ElectionProperties ep;

	private PublicKey pk;

	private int id;

	private Random rand;

	private JRadioButton[][] radioButtons;

	private ButtonGroup[] bg;

	private JPanel bottomLayer;

	private Dimension screenSize;

	private JFrame dialog;

	private JTextField hostField;

	private JSpinner portField;

	private RemoteEntityConnectionC rec;

	private ElectionServer es;

	private Voter c;

	public GUI() {
		super("Voting System!");

		try {
			UIManager
					.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Exception e) {
		}

		this.setSize(800, 600);
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation(screenSize.width / 2 - 400,
				screenSize.height / 2 - 300);
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		rand = new SecureRandom();
		createStartupWindow();
	}

	public void createStartupWindow() {
		dialog = new JFrame();
		dialog.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel dialogPanel = new JPanel(new BorderLayout());
		JPanel inputPanel = new JPanel(new GridLayout(2, 2));
		dialog.setSize(500, 200);

		dialog.setLocation(screenSize.width / 2 - 250,
				screenSize.height / 2 - 100);
		dialog.setVisible(true);
		dialog.setResizable(false);
		JLabel hostLabel = new JLabel("IP/Hostname for Server");
		JLabel portLabel = new JLabel("TCP Port for Server");
		hostField = new JTextField("localhost");
		hostField.setHorizontalAlignment(JTextField.RIGHT);
		hostField.setColumns(20);
		portField = new JSpinner();
		portField.setValue(new Integer("2004"));
		((JSpinner.NumberEditor) portField.getEditor()).getTextField()
				.setColumns(17);

		JPanel hostLabelPanel = new JPanel();
		JPanel hostFieldPanel = new JPanel();
		JPanel portLabelPanel = new JPanel();
		JPanel portFieldPanel = new JPanel();

		hostLabelPanel.add(hostLabel);
		hostFieldPanel.add(hostField);
		portLabelPanel.add(portLabel);
		portFieldPanel.add(portField);

		inputPanel.add(hostLabelPanel);
		inputPanel.add(hostFieldPanel);
		inputPanel.add(portLabelPanel);
		inputPanel.add(portFieldPanel);

		JButton dialogButton = new JButton("Connect");
		dialogButton.addActionListener(this);

		dialogPanel.add(inputPanel);
		dialogPanel.add(dialogButton, BorderLayout.SOUTH);
		dialog.getContentPane().add(dialogPanel);

		dialog.validate();
	}

	public void createGUI() {
		bottomLayer = new JPanel(new BorderLayout());
		JPanel buttonLayer = new JPanel();
		bottomLayer.add(buttonLayer);

		this.getContentPane().add(bottomLayer);

		buttonLayer.setLayout(new GridLayout(ep.getL() + 1,
				ep.getNoOfVotes() + 1));
		bg = new ButtonGroup[ep.getNoOfVotes()];
		radioButtons = new JRadioButton[ep.getNoOfVotes()][ep.getL()];
		for (int i = 0; i < ep.getNoOfVotes(); i++) {
			bg[i] = new ButtonGroup();
		}
		buttonLayer.add(new JLabel());
		for (int i = 1; i < ep.getNoOfVotes() + 1; i++)
			buttonLayer.add(new JLabel("Weight: "+ ep.getVoteWeights()[i-1]));

		for (int i = 0; i < ep.getL(); i++) {
			buttonLayer.add(new JLabel(ep.getVoteNames()[i]));
			for (int j = 0; j < ep.getNoOfVotes(); j++) {
				radioButtons[j][i] = new JRadioButton();
				buttonLayer.add(radioButtons[j][i]);
				bg[j].add(radioButtons[j][i]);
			}
		}
		JButton submit = new JButton("Submit Vote(s)");
		submit.addActionListener(this);
		bottomLayer.add(submit, BorderLayout.SOUTH);
		this.getContentPane().validate();
	}
	private void showVoteCast(){
		bottomLayer.removeAll();
		bottomLayer.add(new JLabel("Vote(s) Cast"));
		bottomLayer.validate();
	}

	public void getElectionParameters() {
		String host = hostField.getText();
		int port = ((Integer) portField.getValue()).intValue();
		System.out.println(host);
		System.out.println(port);
		try {
			rec = new RemoteEntityConnectionC(host, port);
			es = rec.getElectionServer();
			ep = es.getElectionProperties();
			pk = es.getPublicKey();
			id = es.getVoterId();

			dialog.dispose();
		} catch (Exception e) {
			e.printStackTrace();
			showDialog("Could not find server, or access denied!");
		}

		try {
			BulletinBoard bb = rec.getBulletinBoard();
			c = new Voter(id, ep, pk, rand, bb);
		} catch (AccessException e) {
			showDialog("Could not find server, or access denied!");
			e.printStackTrace();
		} catch (RemoteException e) {
			showDialog("Could not find server, or access denied!");
			e.printStackTrace();
		} catch (NotBoundException e) {
			showDialog("Could not find BulletinBoard, or access denied!");
			e.printStackTrace();
		}

	}

	public void submitVotes() {
		int[] candidates = new int[ep.getNoOfVotes()];

		// Get chosen candidates
		for (int i = 0; i < ep.getNoOfVotes(); i++) {
			for (int j = 0; j < ep.getL(); j++) {
				if (radioButtons[i][j].isSelected()) {
					candidates[i] = j;
					System.out.println("vote nr: " + i + " to " + j);
				}
			}
		}

		switch (ep.getElectionType()) {
		case 0:
			try {
				c.postOneofLVote(candidates[0]);
				showVoteCast();
			} catch (IncorrectVoteException e) {
				showDialog("Incorrect Vote");
			} catch (RemoteException e) {
				showDialog("Could not find BulletinBoard, or access denied!");
			}
			break;
		case 1:
			try {
				c.postKofLVote(candidates);
				showVoteCast();
			} catch (IncorrectVoteException e) {
				showDialog("Incorrect Vote");
			} catch (RemoteException e) {
				showDialog("Could not find BulletinBoard, or access denied!");
			}
			break;
		case 2:
			try {
				c.postOneofLVoteBinary(candidates[0]);
				showVoteCast();
			} catch (IncorrectVoteException e) {
				showDialog("Incorrect Vote");
			} catch (RemoteException e) {
				showDialog("Could not find BulletinBoard, or access denied!");
			}
			break;
		case 3:
			try {
				c.postKofLVoteBinary(candidates);
				showVoteCast();
			} catch (IncorrectVoteException e) {
				showDialog("Incorrect Vote");
			} catch (RemoteException e) {
				showDialog("Could not find BulletinBoard, or access denied!");
			}
			break;

		}

	}

	public void showDialog(String msg) {
		JOptionPane.showMessageDialog(this, msg);

	}

	public void actionPerformed(ActionEvent e) {
		System.out.println(e.getActionCommand());
		if (e.getActionCommand().equals("Connect")) {
			getElectionParameters();
			createGUI();
		}
		if (e.getActionCommand().equals("Submit Vote(s)")) {
			submitVotes();
		}
	}

	public static void main(String[] args) {

		// ElectionProperties ep = new ElectionProperties(256, L, 100, 10, 5,
		// 80,1,2, voteNames, 0, null);
		new GUI();
	}

}