package voter;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import remoteInterfaces.BulletinBoard;
import remoteInterfaces.ElectionServer;
/** The class responsible for interacting with the RMI Registry and returning stubs for the remote objects for use in the application.
 * 
 * @author bent
 *
 */
public class RemoteEntityConnectionC {

	private Registry registry;
/** Creates a connection to the RMI Registry at the given host and port
 * 
 * @param host
 * @param port
 * @throws RemoteException If the RMI Registry can not be contacted.
 */
	public RemoteEntityConnectionC(String host, int port)
			throws RemoteException {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new SecurityManager());
		}

		try {
			registry = LocateRegistry.getRegistry(host, port);

		} catch (RemoteException e) {
			throw e;
		}
	}
/** Returns the stub for the remote object ElectionServer
 * 
 * @return The stub for accessing the ElectionServer remote object.
 * @throws AccessException If access is denied
 * @throws RemoteException
 * @throws NotBoundException If the object was not bound to the registry.
 */
	public ElectionServer getElectionServer() throws AccessException,
			RemoteException, NotBoundException {
		ElectionServer obj;

		obj = (ElectionServer) registry.lookup("ElectionServer");

		return obj;
	}
	/** Returns the stub for the remote object BulletinBoard
	 * 
	 * @return The stub for accessing the BulletinBoard remote object.
	 * @throws AccessException If access is denied
	 * @throws RemoteException
	 * @throws NotBoundException If the object was not bound to the registry.
	 */
	public BulletinBoard getBulletinBoard() throws AccessException,
			RemoteException, NotBoundException {
		BulletinBoard obj;

		obj = (BulletinBoard) registry.lookup("BulletinBoard");

		return obj;
	}

}
