package votingSystem;

import java.math.BigInteger;

/**
 * Handles the decryption of ciphertexts in the Paillier cryptosystem. It is not used in the voting system, 
 * as the voting system uses Threshold decryption with multicomputations done by the decryption servers.
 * 
 * @author Bent K. Onshus
 * 
 */
public class Decryption {

	int s, k, noOfVoters;

	BigInteger g, n, d, p, q, phi;

	/**
	 * Constructs a decryptor with the given properties
	 * 
	 * @param k
	 *            Security parameter (bitlength)
	 * @param noOfVoters
	 *            ...
	 * @param s
	 *            ...
	 * @param g
	 *            A generator
	 * @param p
	 *            ...
	 * @param q
	 *            ...
	 */
	public Decryption(int k, int noOfVoters, int s, BigInteger g, BigInteger p,
			BigInteger q) {
		this.k = k;
		this.noOfVoters = noOfVoters;
		this.s = s;
		this.n = p.multiply(q);
		this.g = g;
		this.p = p;
		this.q = q;
		this.phi = p.subtract(BigInteger.ONE).multiply(
				q.subtract(BigInteger.ONE));
		this.d = Utility.lcm(p.subtract(BigInteger.ONE), q
				.subtract(BigInteger.ONE));

		// BigInteger test =
		// ((g.modPow(d,n.pow(s)).subtract(BigInteger.ONE)).divide(n)).modInverse(n);
	}

	public BigInteger calculateL(BigInteger b) {
		BigInteger L = b.subtract(BigInteger.ONE).divide(n);
		System.out.println("L: " + L);
		return L;
	}

	private BigInteger calculateIOptimized(BigInteger c, int s) {
		// Calculate n-values (n^x)
		BigInteger[] nArray = new BigInteger[s + 1];
		nArray[0] = n;
		for (int i = 0; i < s; i++) {
			nArray[i + 1] = nArray[i].multiply(nArray[0]);
		}
		// Find lambda so we can find d.
		BigInteger p1_q1 = (p.subtract(BigInteger.ONE)).multiply(q
				.subtract(BigInteger.ONE)); // (p - 1)(q - 1)
		BigInteger lambda = p1_q1.divide((p.subtract(BigInteger.ONE)).gcd(q
				.subtract(BigInteger.ONE)));

		// Find d so we can decrypt.
		BigInteger d;
		d = lambda.multiply(lambda.modInverse(nArray[s - 1]));

		BigInteger a = c.modPow(d, nArray[s]);
		BigInteger[] lArray = new BigInteger[s];

		for (int j = 0; j < s; j++) {
			lArray[j] = ((a.mod(nArray[j + 1])).subtract(BigInteger.ONE))
					.divide(nArray[0]);
		}

		BigInteger[] iArray = new BigInteger[s + 1];

		iArray[0] = BigInteger.ZERO;

		BigInteger counter = BigInteger.ONE;
		BigInteger faculty[] = new BigInteger[s];
		BigInteger temp = BigInteger.ONE;
		for (int i = 1; i < s; i++) {
			// faculty[i] = n^{i+1} / (i+2)! mod n^s (n^s, so it can be reduced
			// to all n^i)
			counter = counter.add(BigInteger.ONE);
			temp = temp.multiply(counter);
			faculty[i - 1] = (nArray[i - 1].multiply(temp
					.modInverse(nArray[s - 1]))).mod(nArray[s - 1]);
		}

		for (int j = 1; j <= s; j++) {
			BigInteger t1 = lArray[j - 1];
			BigInteger t2 = iArray[j - 1];
			for (int k = 2; k <= j; k++) {
				iArray[j - 1] = iArray[j - 1].subtract(BigInteger.ONE);
				t2 = (t2.multiply(iArray[j - 1])).mod(nArray[j - 1]);
				t1 = (t1.subtract(t2
						.multiply(faculty[k - 2].mod(nArray[j - 1]))))
						.mod(nArray[j - 1]);
				System.out.println("I andre loop");
			}
			iArray[j] = t1;

		}
		return iArray[s];

	}

	/**
	 * Decrypts the ciphertext c
	 * 
	 * @param c
	 *            The ciphertext
	 * @param s
	 *            ...
	 * @return The plaintext
	 */
	public BigInteger decryption(BigInteger c, int s) {
		BigInteger i = calculateIOptimized(c, s);

		return i;
	}
}
