package votingSystem;

import java.math.BigInteger;
import java.util.Random;

/**
 * This class handles the encryption process in the generalized Paillier
 * cryptosystem used in the voting system. This class is not used in the Voting system, as the Voter class contains
 * and optimized variant of the encryption algorithm.
 * 
 * @author Bent K. Onshus
 * 
 */
public class Encryption {

	BigInteger n, g; // Public Key(n,g)

	Random r;

	int M, L;

	int k;

	int id = 3;

	int s;

	/**
	 * Constructs the Encryption object
	 * 
	 * @param k
	 *            Security parameter (bitlength)
	 * @param noOfVoters
	 *            Number of voters
	 * @param s
	 *            The s in n^s
	 * @param n
	 *            p*q
	 * @param g
	 *            A generator of the group n^s
	 */
	public Encryption(int k, int noOfVoters, int s, BigInteger n, BigInteger g) {
		this.k = k;

		M = noOfVoters;
		L = s;
		this.s = s;
		this.n = n;
		this.g = g;

	}

	public void testEncryption() {
		BigInteger r_ = new BigInteger(k, r);
		BigInteger a[] = new BigInteger[L];
		a[0] = BigInteger.ZERO;
		a[1] = BigInteger.ONE;
		a[2] = BigInteger.ZERO;
		a[3] = BigInteger.ZERO;
		a[4] = BigInteger.ZERO;
		System.out.println(encryptSmall(a, r_));

	}

	public BigInteger[] preCompGenPailler() {
		BigInteger[] temp = new BigInteger[s];
		for (long j = 0; j < s; j++) {
			BigInteger j_fac = BigInteger.ZERO;
			// Optimize this!
			j_fac = Utility.longToBigInt(Utility.faculty(j));

			temp[(int) j] = j_fac.modInverse(n.pow(s + 1));
			temp[(int) j] = temp[(int) j].multiply(n.pow((int) j)).mod(
					n.pow(s + 1));

		}
		return temp;
	}

	/**
	 * Encrypts a vote i_ with random number r
	 * 
	 * @param r
	 *            Random number
	 * @param i_
	 *            The vote
	 * @param s
	 *            The s in n^s
	 * @return the ciphertext
	 */
	public BigInteger encryptGenPailler(BigInteger r, BigInteger i_, int s) {

		BigInteger c = (n.multiply(i_)).add(BigInteger.ONE);// The would-be
															// ciphertext
		BigInteger temp = i_;
		BigInteger precomp = i_;
		BigInteger nj = n;
		BigInteger ns1 = n.pow(s + 1);
		BigInteger ns = n.pow(s);
		for (int j = 2; j <= s; j++) {
			nj = nj.multiply(n);
			temp = temp.subtract(BigInteger.ONE);
			precomp = ((precomp.multiply(temp)).multiply(Utility.intToBigInt(j)
					.modInverse(ns1))).mod(ns1);
			c = (c.add(precomp.multiply(nj))).mod(ns1);
		}
		c = (c.multiply(r.modPow(ns, ns1))).mod(ns1);
		return c;
	}

	public BigInteger encryptGenPaillerSimple(BigInteger r, BigInteger i_, int s) {
		BigInteger ns1 = n.pow(s + 1);
		BigInteger ns = n.pow(s);
		BigInteger c = (n.add(BigInteger.ONE)).modPow(i_, ns1).multiply(
				r.modPow(ns, ns1)).mod(ns1);
		return c;
	}

	// Se 6.1 Damgård, Jurik etc.
	public BigInteger encryptSmall(BigInteger[] a, BigInteger r) {
		BigInteger cipherText = BigInteger.ZERO;

		for (int j = 0; j < L; j++) {
			BigInteger temp = a[j].multiply(Utility.intToBigInt(M).pow(j));
			temp = temp.mod(n.pow(s));
			cipherText = cipherText.add(temp);
		}

		return cipherText;
	}

}
