package votingSystem;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * This class implements the HashFunction used in the voting system. The standard choice of hash function in this system
 * is the SHA-256 implementation available in the standard Java RE.
 * 
 * @author Bent K. Onshus
 * 
 */
public class HashFunction {

	MessageDigest h;

	/**
	 * Constructs the hash function
	 * 
	 * @param instance
	 *            The type of hash function (i.e. SHA-1, SHA256, SHA512 etc..)
	 */
	public HashFunction(String instance) {
		try {
			h = MessageDigest.getInstance(instance);
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Generates a hash from the BigIntegers
	 * 
	 * @param values
	 *            An array containing the BigIntegers to be hashed
	 * @return The hash.
	 */
	public BigInteger generateHash(BigInteger[] values, int t) {

		for (int i = 0; i < values.length; i++) {
			h.update(values[i].toByteArray());
		}
		byte[] digest = h.digest();

		int noOfBytes = t / 8;
		byte[] digestTrunc = new byte[noOfBytes];
		for (int i = 0; i < noOfBytes; i++)
			digestTrunc[i] = digest[i];

		return new BigInteger(digestTrunc);
	}

}
