package votingSystem;

import java.math.BigInteger;
import java.util.Random;

/**
 * This class will be used to generate the keys for the decryption servers, as well as the public key for the election.
 * 
 * @author Bent
 * 
 */
public class KeyGenerator {
	private boolean DEBUGGING = false;

	private Random r;

	private int s;

	private BigInteger d, n, m, v;

	private BigInteger p, q;

	private BigInteger g;

	private BigInteger[] vArray;

	private PublicKey pk;

	/**
	 * This constructor will make a new KeyGenerator with the Random-generator r
	 * 
	 * @param r A Random generator
	 * @param bitlength The bitlength of the key
	 * @param s The exponent used in the cryptosystem.
	 */
	public KeyGenerator(Random r, int bitlength, int s) {

		long time1 = System.currentTimeMillis();
		this.r = r;
		this.s = s;
		BigInteger[] pArray = generateSafePrime(bitlength / 2 - 1);
		BigInteger[] qArray = generateSafePrime(bitlength / 2 - 1);

		if (DEBUGGING) {
			long time2 = System.currentTimeMillis();
			System.out.println("Finished generating Primes after "
					+ (time2 - time1) + " ms");
		}
		BigInteger p = pArray[1];
		BigInteger p_ = pArray[0];

		BigInteger q = qArray[1];
		BigInteger q_ = qArray[0];
		this.p = p;
		this.q = q;
		n = p.multiply(q);
		m = p_.multiply(q_);
		if (DEBUGGING) {
			long time2 = System.currentTimeMillis();
			System.out.println("Finished finding n & m after "
					+ (time2 - time1) + " ms");
		}

		BigInteger ns1 = n.pow(s + 1);
		BigInteger ns = n.pow(s);
		/*
		 * Find the d for use in decryption, by using chinese reaminder on d = 0
		 * mod lambda, d = i^{-1} mod n^{npower-1}
		 */

		BigInteger temp1 = BigInteger.ONE;

		BigInteger temp2 = n.add(BigInteger.ONE);
		g = temp2.modPow(temp1, ns1);
		if (DEBUGGING) {
			long time2 = System.currentTimeMillis();
			System.out.println("Found g after " + (time2 - time1) + " ms");
		}
		// temp1 = new BigInteger(ns1.bitLength(), r).mod(ns1); //??
		temp1 = temp1.modInverse(ns1);
		temp2 = m.modInverse(ns);
		temp1 = temp1.multiply(m);
		temp1 = temp1.multiply(temp2);
		BigInteger ns_m = ns.multiply(m);
		d = temp1.mod(ns_m);
		if (DEBUGGING) {
			long time2 = System.currentTimeMillis();
			System.out.println("Calcluated d after " + (time2 - time1) + " ms");
		}

		BigInteger temp = new BigInteger(ns1.bitLength(), r).mod(ns1);


		v = (temp.multiply(temp)).mod(ns1);
		if (DEBUGGING) {
			long time2 = System.currentTimeMillis();
			System.out.println("Finished with v after " + (time2 - time1)
					+ " ms");
		}

	}
/** Generates the KeyShares for the decryption servers
 * 
 * @param l Number of decryption servers
 * @param w Number of decryption servers needed to perform decryption
 * @return The array of KeyShares for all servers.
 */
	public KeyShare[] generateKeys(int l, int w) {

		BigInteger ns_m = (n.pow(s)).multiply(m);

		BigInteger[] aArray = new BigInteger[w];
		aArray[0] = d;
		for (int i = 1; i < w; i++) {
			aArray[i] = new BigInteger(ns_m.bitLength(), r).mod(ns_m);
			/*
			 * while (ns_m.compareTo (aArray[i]) <= 0) aArray[i] = new
			 * BigInteger (ns_m.bitLength (), r);
			 */
		}

		BigInteger[] sArray = new BigInteger[l];

		/*
		 * BigInteger[][] X = new BigInteger[l+1][w]; for (int i = 0; i <= l;
		 * i++) { X[i][0] = BigInteger.ONE; X[i][1] = new BigInteger("" + i); }
		 * 
		 * for (int i = 2; i < w; i++) { for (int j = 0; j <= l; j++){ X[j][i] =
		 * (X[j][i-1].multiply (X[j][1])).mod (ns_m); }
		 *  } for(int i=0 ; i<l ; i++){ BigInteger temp = aArray[0]; for(int
		 * j=1 ; j<w ; j++){ temp =
		 * (temp.add(aArray[j].multiply(X[i+1][j]))).mod(ns_m); } sArray[i] =
		 * temp; }
		 */

		for (int i = 0; i < l; i++) {
			BigInteger temp = BigInteger.ZERO;
			for (int j = 0; j < w; j++) {
				temp = temp.add(
						aArray[j].multiply(BigInteger.valueOf(i + 1).pow(j))
								.mod(ns_m)).mod(ns_m);
			}
			sArray[i] = temp;
		}

		// Generate verfication keys
		vArray = new BigInteger[l];
		BigInteger delta = Utility.intToBigInt((int) Utility.faculty(l));
		for (int i = 0; i < l; i++) {
			vArray[i] = v.modPow(delta.multiply(sArray[i]), n.pow(s + 1));
		}

		KeyShare[] ks = new KeyShare[l];

		for (int i = 0; i < l; i++) {
			ks[i] = new KeyShare(i, v, vArray[i], sArray[i], n, s);
		}

		pk = new PublicKey(n, g, v, vArray, s);
		return ks;
	}

	/**
	 * Generates a safe prime (p = 2p' + 1) where p' is a prime with 1-2^100
	 * probability
	 * 
	 * @param bitlength
	 *            The bitlength of the primes to be generated
	 * @return The numbers p and p'
	 */
	public BigInteger[] generateSafePrime(int bitlength) {
		BigInteger p[] = new BigInteger[2];
		// p[0] is p', p[1] is p
		p[0] = BigInteger.ZERO;
		p[1] = BigInteger.ZERO;
		int counter = 1;
		while (!p[1].isProbablePrime(50)) {
			p[0] = BigInteger.probablePrime(bitlength, r);
			p[1] = (p[0].multiply(BigInteger.valueOf(2))).add(BigInteger.ONE);
			counter++;
		}
		System.out.println("Number of tries needed: " + counter);
		return p;
	}

	public BigInteger getD() {
		return d;
	}

	public BigInteger getM() {
		return m;
	}

	public BigInteger getN() {
		return n;
	}

	public Random getR() {
		return r;
	}

	public int getS() {
		return s;
	}

	public BigInteger getV() {
		return v;
	}

	public BigInteger getG() {
		return g;
	}

	public BigInteger getP() {
		return p;
	}

	public BigInteger getQ() {
		return q;
	}

	public PublicKey getPk() {
		return pk;
	}

}