package votingSystem;

import java.io.Serializable;
import java.math.BigInteger;

/**
 * The class specifies the Data Structure for holding the KeyShares for the decryption Servers
 * 
 * @author Bent K. Onshus
 * 
 */
public class KeyShare implements Serializable {

	private BigInteger v, vi, si, n;

	private int s, id;

	/**
	 * Creates a new KeyShare
	 * 
	 * @param v
	 *            The public base for the verification keys
	 * @param vi
	 *            The verification key for this Server
	 * @param si
	 *            The secret keyshare for this Server
	 * @param n
	 *            The public value n = p*q
	 * @param s
	 *            The public cryptosystem exponent s (n^s)
	 */
	public KeyShare(int id, BigInteger v, BigInteger vi, BigInteger si,
			BigInteger n, int s) {
		this.id = id;
		this.v = v;
		this.vi = vi;
		this.si = si;
		this.s = s;
		this.n = n;
	}

	public BigInteger getN() {
		return n;
	}

	public int getS() {
		return s;
	}

	public BigInteger getSi() {
		return si;
	}

	public BigInteger getV() {
		return v;
	}

	public BigInteger getVi() {
		return vi;
	}

	public int getId() {
		return id;
	}

}
