package votingSystem;

import java.io.Serializable;
import java.math.BigInteger;

/**
 * The class is the data structure for holding the PublicKey for the election.
 * 
 * @author Bent K. Onshus
 * 
 */
public class PublicKey implements Serializable {

	private BigInteger g, n, v;

	private BigInteger[] vi, nArray;

	private int s;

	/**
	 * Creates a new PublicKey for the election
	 * 
	 * @param n
	 *            Public value n
	 * @param g
	 *            Generator g for use in the encryptions
	 * @param vi
	 *            The list of verification keys for the Servers
	 */
	public PublicKey(BigInteger n, BigInteger g, BigInteger v, BigInteger[] vi,
			int s) {
		this.s = s;
		this.n = n;
		this.v = v;
		this.g = g;
		this.vi = vi;
		nArray = new BigInteger[s + 2];
		nArray[0] = BigInteger.ONE;
		for (int i = 1; i <= s + 1; i++) {
			nArray[i] = nArray[i - 1].multiply(n);
		}

	}

	public BigInteger getG() {
		return g;
	}

	public BigInteger getN() {
		return n;
	}

	public BigInteger[] getVi() {
		return vi;
	}

	public int getS() {
		return s;
	}

	public BigInteger[] getNArray() {
		return nArray;
	}

	public BigInteger getV() {
		return v;
	}

}
