package votingSystem;

import java.math.BigInteger;
/** This class provides the methods used for validation of Votes
 * 
 * @author Bent K. Onshus
 *
 */
public class Validator {

	private ElectionProperties ep;

	private PublicKey pk;

	private BigInteger n;

	private BigInteger[] voteValues;

	private int k;

	private int s;

	private int M;

	private int noOfVotes;

	private int t;

	private BigInteger g; 
	
	private int L;

	private int l;
	
	private BigInteger[] nArray;

	private BigInteger[] binaryVoteValues,normalVoteValues;;
	
	private HashFunction h;

	private BigInteger v;
	
	private BigInteger pow_t;
	
	private BigInteger gInverse;
	
	private BigInteger precomp[];
/** Creates a Validator for the given ElectionProperties and PublicKey
 * 
 * @param ep The ElectionProperties used in the election
 * @param pk The PublicKey for the election
 */
	public Validator(ElectionProperties ep, PublicKey pk) {
		this.ep = ep;
		this.pk = pk;
		this.n = pk.getN();
		this.M = ep.getM();
		this.s = pk.getS();
		this.k = ep.getK();
		this.t = ep.getT();
		this.noOfVotes = ep.getNoOfVotes();
		this.voteValues = ep.getVoteValues();
		this.nArray = pk.getNArray();
		this.h = new HashFunction("SHA-256");
		this.v = pk.getV();
		this.L = ep.getL();
		this.precomp = this.preCompute();
		this.g = pk.getG();
		this.gInverse = g.modInverse(nArray[s + 1]);
		pow_t = BigInteger.valueOf(2).pow(t);
	
		this.l = 0;
		while (Math.pow(2, l + 1) < L)
			l++;
		
		int type = ep.getElectionType();
		if(type ==2 || type ==3){
		this.binaryVoteValues = this.computeVoteValuesBin();
		}
		else{
		this.normalVoteValues = this.computeVoteValuesNorm();
		}
	}

	private BigInteger[] computeVoteValuesNorm(){
		BigInteger voteValueArray[] = new BigInteger[L];
		for(int i=0 ; i < L ; i++){
			voteValueArray[i] = (g.modPow(voteValues[i], nArray[s + 1]))
			.modInverse(nArray[s + 1]);
		}
		return voteValueArray;
			
	}
	
	private  BigInteger[] computeVoteValuesBin(){
		BigInteger tempVoteValue = BigInteger.valueOf(M);
		BigInteger voteValueArray[] = new BigInteger[l+1];
		for(int i=0 ; i <= l ; i++){
			voteValueArray[i] = (g.modPow(tempVoteValue, nArray[s + 1]))
			.modInverse(nArray[s + 1]);
			tempVoteValue = tempVoteValue.pow(2);
		}
		return voteValueArray;
			
	}
	/**
	 * The code for checking if a BinaryVote is of the correct form.
	 * 
	 * @param vote The BinaryVote to be checked for correctness
	 *            
	 * @return true if the proof is correct.
	 */
	public boolean checkBinaryVote(BinaryVote vote) {
		BigInteger[] encryption = vote.getEncryption();
		BigInteger[][] e = vote.getE();
		BigInteger[][] z = vote.getZ();
		BigInteger[][] a = vote.getA();


		// The array that holds the values to be used in the hash-function
		BigInteger valuesToBeHashed[] = new BigInteger[4];
		
		valuesToBeHashed[0] = n;
		valuesToBeHashed[1] = BigInteger.valueOf(vote.getId());

		BigInteger u1,u2, voteCheckValue1, voteCheckValue2;
		
		
		for (int j = 0; j <= l; j++) {

			// Check all e,a,z values to see if they are correct.

			u1 = encryption[j].multiply(binaryVoteValues[j]);
			 voteCheckValue1 = a[0][j].multiply(
					u1.modPow(e[0][j], nArray[s + 1])).mod(nArray[s + 1]);
			 voteCheckValue2 = z[0][j].modPow(nArray[s],
					nArray[s + 1]);

			if (voteCheckValue1.compareTo(voteCheckValue2) != 0)
				return false;

			 u2 = encryption[j].multiply(gInverse);
			voteCheckValue1 = a[1][j].multiply(
					u2.modPow(e[1][j], nArray[s + 1])).mod(nArray[s + 1]);
			voteCheckValue2 = z[1][j].modPow(nArray[s], nArray[s + 1]);

			if (voteCheckValue1.compareTo(voteCheckValue2) != 0)
				return false;

			if ((n.gcd(a[0][j])).compareTo(BigInteger.ONE) > 0
					|| (n.gcd(z[0][j])).compareTo(BigInteger.ONE) > 0
					|| (n.gcd(a[1][j])).compareTo(BigInteger.ONE) > 0
					|| (n.gcd(z[1][j])).compareTo(BigInteger.ONE) > 0)
				return false;
			// Save a_i in the array to hold values to be hashed.
			valuesToBeHashed[2] = a[0][j];
			valuesToBeHashed[3] = a[1][j];

			// Check if the sum of e_i values are correct!

			voteCheckValue1 = h.generateHash(valuesToBeHashed, t).mod(pow_t);

			voteCheckValue2 = e[0][j].add(e[1][j]).mod(pow_t);
			
			if (voteCheckValue1.compareTo(voteCheckValue2) != 0)
				return false;
			
		
		}

		// Now it's time to check the multiplication proofs.

		BigInteger[] fi = vote.getFi();
		BigInteger[] f = vote.getF();
		BigInteger[] ed = vote.getEd();
		BigInteger[] edb = vote.getEdb();
		BigInteger[] z1 = vote.getZ1();
		BigInteger[] z2 = vote.getZ2();
		BigInteger[] eMult = new BigInteger[f.length];

		// We set f[0] = e[0] to ensure that the calculations start from a
		// correct value.
		fi[0] = encryption[0];
		BigInteger[] valuesToBeHashed2 = new BigInteger[5];
		valuesToBeHashed2[0] = n;
		valuesToBeHashed2[4] = BigInteger.valueOf(vote.getId());
		for (int i = 0; i < l; i++) {

			valuesToBeHashed2[1] = fi[i];
			valuesToBeHashed2[2] = encryption[i + 1];
			valuesToBeHashed2[3] = fi[i + 1];
			eMult[i] = h.generateHash(valuesToBeHashed2, t);
			voteCheckValue1 = ((fi[i].modPow(eMult[i], nArray[s + 1]))
					.multiply(ed[i])).mod(nArray[s + 1]);
			voteCheckValue2 = encryptCheck(f[i], z1[i]);

			if (voteCheckValue1.compareTo(voteCheckValue2) != 0)
				return false;

			voteCheckValue1 = edb[i].multiply(fi[i + 1].modPow(eMult[i],
					nArray[s + 1]));
			voteCheckValue1 = voteCheckValue1.modInverse(nArray[s + 1]);
			voteCheckValue1 = (voteCheckValue1.multiply(encryption[i + 1].modPow(f[i],
					nArray[s + 1]))).mod(n);
			voteCheckValue2 = encryptCheck(BigInteger.ZERO, z2[i]).mod(n);

			if (voteCheckValue1.compareTo(voteCheckValue2) != 0)
				return false;
		}

		return true;
	}
	/**
	 * The code for checking if the MultipleBinaryVotes are of the correct form.
	 * 
	 * @param mVotes The MultipleBinaryVotes-object to be checked for correctness
	 *            
	 * @return true if the proof is correct.
	 */
	public boolean checkMultipleBinaryVote(MultipleBinaryVotes mVotes) {
		BinaryVote[] votes = mVotes.getVotes();
		BigInteger ea[][] = mVotes.getEa();
		BigInteger eb[][] = mVotes.getEb();
		BigInteger ec[][] = mVotes.getEc();
		BigInteger ed[][] = mVotes.getEd();
		BigInteger edb[][] = mVotes.getEdb();
		BigInteger f[][] = mVotes.getF();
		BigInteger e[][] = mVotes.getE();
		BigInteger z1[][] = mVotes.getZ1();
		BigInteger z2[][] = mVotes.getZ2();
		int clientId = mVotes.getId();

		if (ea.length != noOfVotes || ea[0].length != noOfVotes
				|| eb.length != noOfVotes || eb[0].length != noOfVotes
				|| ed.length != noOfVotes || ed[0].length != noOfVotes
				|| edb.length != noOfVotes || edb[0].length != noOfVotes
				|| f.length != noOfVotes || f[0].length != noOfVotes
				|| z1.length != noOfVotes || z1[0].length != noOfVotes) {
			return false;
		}
		int[] validVotesIndex = new int[votes.length];
		for (int i = 0; i < votes.length; i++) {
			if (checkBinaryVote(votes[i])) {
				validVotesIndex[i] = 1;
			} else {
				validVotesIndex[i] = 0;
			}

		}
		
		int l = votes[0].getFi().length-1;
		BigInteger[] valuesToBeHashed = new BigInteger[5];
		valuesToBeHashed[0] = pk.getN();
		valuesToBeHashed[4] = BigInteger.valueOf(clientId);
		
		BigInteger temp_, temp1, temp2, voteDiff;
		
		for (int i = 0; i < votes.length; i++) {
			//Skip the vote if it is not valid
			if (validVotesIndex[i] == 0) {
				continue;
			}
			for (int j = i + 1; j < votes.length; j++) {
				//Skip the vote if it is not valid
				if (validVotesIndex[j] == 0) {
					continue;
				}

				valuesToBeHashed[1] = ea[i][j];
				valuesToBeHashed[2] = eb[i][j];
				valuesToBeHashed[3] = ec[i][j];

				e[i][j] = h.generateHash(valuesToBeHashed, t);

				temp_ = votes[j].getFi()[l]
						.modInverse(nArray[s + 1]);
				voteDiff = ((votes[i].getFi()[l])
						.multiply(temp_)).mod(nArray[s + 1]);

				if (voteDiff.compareTo(ea[i][j]) != 0)
					return false;

				temp1 = ((ea[i][j].modPow(e[i][j], nArray[s + 1]))
						.multiply(ed[i][j])).mod(nArray[s + 1]);
				temp2 = encryptCheck(f[i][j], z1[i][j]);

				if (temp1.compareTo(temp2) != 0)
					return false;

				temp1 = (edb[i][j].multiply(ec[i][j].modPow(e[i][j],
						nArray[s + 1]))).modInverse(nArray[s + 1]);
				temp1 = temp1.multiply(eb[i][j].modPow(f[i][j], nArray[s + 1]))
						.mod(nArray[s + 1]);
				temp2 = encryptCheck(BigInteger.ZERO, z2[i][j]);

				if (temp1.compareTo(temp2) != 0)
					return false;

			}
		}
		return true;

	}
	/**
	 * The code for checking if MultipleVotes is of the correct form.
	 * 
	 * @param mVotes The MultipleVotes to be checked for correctness
	 *            
	 * @return true if the proof is correct.
	 */
	public boolean checkMultipleVote(MultipleVotes mVotes) {
		Vote[] votes = mVotes.getVotes();
		BigInteger ea[][] = mVotes.getEa();
		BigInteger eb[][] = mVotes.getEb();
		BigInteger ec[][] = mVotes.getEc();
		BigInteger ed[][] = mVotes.getEd();
		BigInteger edb[][] = mVotes.getEdb();
		BigInteger f[][] = mVotes.getF();
		BigInteger e[][] = mVotes.getE();
		BigInteger z1[][] = mVotes.getZ1();
		BigInteger z2[][] = mVotes.getZ2();
		int clientId = mVotes.getId();

		if (ea.length != noOfVotes || ea[0].length != noOfVotes
				|| eb.length != noOfVotes || eb[0].length != noOfVotes
				|| ed.length != noOfVotes || ed[0].length != noOfVotes
				|| edb.length != noOfVotes || edb[0].length != noOfVotes
				|| f.length != noOfVotes || f[0].length != noOfVotes
				|| z1.length != noOfVotes || z1[0].length != noOfVotes) {
			return false;
		}
		int[] validVotesIndex = new int[votes.length];
		for (int i = 0; i < votes.length; i++) {
			if (checkProof(votes[i])) {
				validVotesIndex[i] = 1;
			} else {
				validVotesIndex[i] = 0;
			}

		}

		BigInteger[] valuesToBeHashed = new BigInteger[5];
		valuesToBeHashed[0] = n;
		valuesToBeHashed[4] = BigInteger.valueOf(clientId);
		
		BigInteger voteDiff, temp1, temp2;
		
		for (int i = 0; i < votes.length; i++) {
			if (validVotesIndex[i] == 0) {
				continue;
			}
			for (int j = i + 1; j < votes.length; j++) {
				if (validVotesIndex[j] == 0) {
					continue;
				}



				valuesToBeHashed[1] = ea[i][j];
				valuesToBeHashed[2] = eb[i][j];
				valuesToBeHashed[3] = ec[i][j];

				e[i][j] = h.generateHash(valuesToBeHashed, t);

				voteDiff = ((votes[i].getVote()).multiply(votes[j]
						.getVote().modInverse(nArray[s + 1])))
						.mod(nArray[s + 1]);
				if (voteDiff.compareTo(ea[i][j]) != 0)
					return false;

				temp1 = ((ea[i][j].modPow(e[i][j], nArray[s + 1]))
						.multiply(ed[i][j])).mod(nArray[s + 1]);
				temp2 = encryptCheck(f[i][j], z1[i][j]);

				if (temp1.compareTo(temp2) != 0)
					return false;

				temp1 = (edb[i][j].multiply(ec[i][j].modPow(e[i][j],
						nArray[s + 1]))).modInverse(nArray[s + 1]);
				temp1 = temp1.multiply(eb[i][j].modPow(f[i][j], nArray[s + 1]))
						.mod(nArray[s + 1]);
				temp2 = encryptCheck(BigInteger.ZERO, z2[i][j]);

				if (temp1.compareTo(temp2) != 0)
					return false;
			}
		}
		return true;

	}

	/**
	 * The code for checking if a Vote is of the correct form.
	 * 
	 * @param v The Vote to be checked for correctness
	 *            
	 * @return true if the proof is correct.
	 */
	public boolean checkProof(Vote v) {

		BigInteger[] e = v.getE();
		BigInteger[] z = v.getZ();
		BigInteger[] a = v.getA();
		BigInteger e_challenge = v.getChallenge();
		// The variable to hold the sum of e_i values
		BigInteger e_sum = BigInteger.ZERO;

		// The array that holds the values to be used in the hash-function
		BigInteger valuesToBeHashed[] = new BigInteger[L + 2];
		valuesToBeHashed[0] = n;
		valuesToBeHashed[1] = BigInteger.valueOf(v.getId());

		BigInteger voteCheckValue1, voteCheckValue2;
		
		
		// Check all e,a,z values to see if they are correct.
		for (int i = 0; i < L; i++) {
			e_sum = e_sum.add(e[i]).mod(pow_t);

			voteCheckValue1 = normalVoteValues[i].multiply(v.getVote());
			voteCheckValue1 = a[i].multiply(
					voteCheckValue1.modPow(e[i], nArray[s + 1])).mod(
					nArray[s + 1]);

			voteCheckValue2 = z[i].modPow(nArray[s], nArray[s + 1]);

			if (voteCheckValue1.compareTo(voteCheckValue2) != 0)
				return false;

			if ((n.gcd(a[i])).compareTo(BigInteger.ONE) > 0
					|| (n.gcd(z[i])).compareTo(BigInteger.ONE) > 0)
				return false;
			// Save a_i in the array to hold values to be hashed.
			valuesToBeHashed[i + 2] = a[i];
		}
		// Check if the sum of e_i values are correct!
		BigInteger e_hash = h.generateHash(valuesToBeHashed, t).mod(pow_t);

		if (e_hash.compareTo(e_sum) != 0)
			return false;

		return true;

	}

	/** Method for precomputing values used in the optimized encryption algorithm
	 * 
	 * @return
	 */
		private BigInteger[] preCompute() {
			BigInteger precomp[] = new BigInteger[s + 1];
			BigInteger jFac = BigInteger.ONE;
			precomp[1] = n;
			for (int j = 1; j <= s; j++) {
				jFac = jFac.multiply(BigInteger.valueOf(j));
				precomp[j] = jFac.modInverse(nArray[s + 1]);
				precomp[j] = precomp[j].multiply(n.pow(j)).mod(nArray[s + 1]);
			}
			return precomp;
		}
	
	public BigInteger encryptCheckUnOptimized(BigInteger value, BigInteger random) {

		BigInteger temp1 = pk.getG().modPow(value, nArray[s + 1]);
		BigInteger temp2 = random.modPow(nArray[s], nArray[s + 1]);

		BigInteger ciphertext_vote = (temp1.multiply(temp2)).mod(nArray[s + 1]);

		return ciphertext_vote;

	}

	/** Optimized encryption algorithm for encrypting a BigInteger with the given random value 
	 * Mainly used when checking the zero-knowledge proofs.
	 * 
	 * @param value
	 * @return The ciphertext and the random value used to hide the plaintext
	 */
		public BigInteger encryptCheck(BigInteger value, BigInteger random) {
	
			BigInteger c = BigInteger.ONE.add(value.multiply(n));
			BigInteger temp = value;
			for (int j = 2; j <= s; j++) {
				BigInteger j_ = BigInteger.valueOf(j);
				temp = temp.multiply((value.subtract(j_)).add(BigInteger.ONE)).mod(
						nArray[s - j + 1]);
				c = (c.add(temp.multiply(precomp[j]))).mod(nArray[s + 1]);
			}

			BigInteger rns = random.modPow(nArray[s], nArray[s + 1]);
			return c.multiply(rns).mod(nArray[s + 1]);
	
		}
	
	
	/**
	 * This method verifies the zero-knowledge proof for the decrypted shares
	 * acquired from the decryption Servers
	 * 
	 * @param ds
	 *            The DecryptedShare to be verified
	 * @return true if the DecryptedShare is correct.
	 */
	public boolean checkShareDecryption(DecryptedShare ds) {
		BigInteger[] vi = pk.getVi();

		BigInteger e = ds.getE();
		BigInteger u = (ds.getC().pow(4)).mod(nArray[s + 1]);
		BigInteger u_tilde = (ds.getCi().pow(2)).mod(nArray[s + 1]);

		BigInteger uz = u.modPow(ds.getZ(), nArray[s + 1]);
		BigInteger u_tilde_e = u_tilde.modPow(e, nArray[s + 1]);
		u_tilde_e = u_tilde_e.modInverse(nArray[s + 1]).mod(nArray[s + 1]);
		BigInteger check_1 = uz.multiply(u_tilde_e).mod(nArray[s + 1]);

		BigInteger v_tilde = vi[ds.getTallyId()];
		BigInteger vz = v.modPow(ds.getZ(), nArray[s + 1]);
		BigInteger v_tilde_e = v_tilde.modPow(e, nArray[s + 1]);
		v_tilde_e = v_tilde_e.modInverse(nArray[s + 1]).mod(nArray[s + 1]);
		BigInteger check_2 = vz.multiply(v_tilde_e).mod(nArray[s + 1]);

		BigInteger[] valuesToBeHashed = new BigInteger[5];
		valuesToBeHashed[0] = n;
		valuesToBeHashed[1] = check_1;
		valuesToBeHashed[2] = check_2;
		valuesToBeHashed[3] = u;
		valuesToBeHashed[4] = u_tilde;

		BigInteger digest = h.generateHash(valuesToBeHashed, ep.getT());

		if (digest.compareTo(e) != 0)
			return false;

		return true;

	}
}
